# Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
`namespace ` [`opcua`](#namespaceopcua) | opcua is used as enclosing namespace for all parts of the Arduino_OPC_UA library in order to avoid naming conflicts with already existing frameworks pertaining the Arduino [Opta](#classopcua_1_1_opta).

# namespace `opcua` <a id="namespaceopcua" class="anchor"></a>

opcua is used as enclosing namespace for all parts of the Arduino_OPC_UA library in order to avoid naming conflicts with already existing frameworks pertaining the Arduino [Opta](#classopcua_1_1_opta).

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
`public time_t ` [`timeToStr`](#namespaceopcua_1a5eab7edc9a71d850d6b47ae50e214c08)`(char const * time)`            | Converts a date/time string as generated by **DATE** into a time_t value. 
`class ` [`opcua::AnalogExpansion`](#classopcua_1_1_analog_expansion) | Provides an OPC UA object node abstraction for an Analog [Expansion](#classopcua_1_1_expansion) board.
`class ` [`opcua::DigitalExpansion`](#classopcua_1_1_digital_expansion) | Provides an OPC UA object node abstraction for both Digital [Expansion](#classopcua_1_1_expansion) board.
`class ` [`opcua::DigitalMechExpansion`](#classopcua_1_1_digital_mech_expansion) | Provides an OPC UA object node abstraction for a Digital [Expansion](#classopcua_1_1_expansion) board with mechanical relays.
`class ` [`opcua::DigitalStSolidExpansion`](#classopcua_1_1_digital_st_solid_expansion) | Provides an OPC UA object node abstraction for a Digital [Expansion](#classopcua_1_1_expansion) board with solid-state relays.
`class ` [`opcua::Expansion`](#classopcua_1_1_expansion) | Provides a generic base class for various expansion board specificic derived classes.
`class ` [`opcua::AnalogInput`](#classopcua_1_1_analog_input) | Provides an OPC UA variable node abstraction for an analog input.
`class ` [`opcua::AnalogInputManager`](#classopcua_1_1_analog_input_manager) | Provides methods for creating and storing [AnalogInput](#classopcua_1_1_analog_input) objects in a collection.
`class ` [`opcua::AnalogOutput`](#classopcua_1_1_analog_output) | Provides an OPC UA variable node abstraction for an analog output.
`class ` [`opcua::AnalogOutputManager`](#classopcua_1_1_analog_output_manager) | Provides methods for creating and storing [AnalogOutput](#classopcua_1_1_analog_output) objects in a collection.
`class ` [`opcua::UserButton`](#classopcua_1_1_user_button) | Provides an OPC UA variable node abstraction for a user button input.
`class ` [`opcua::DigitalInput`](#classopcua_1_1_digital_input) | Provides an OPC UA variable node abstraction for a digital input.
`class ` [`opcua::DigitalInputManager`](#classopcua_1_1_digital_input_manager) | Provides methods for creating and storing [DigitalInput](#classopcua_1_1_digital_input) objects in a collection.
`class ` [`opcua::Led`](#classopcua_1_1_led) | Provides an OPC UA variable node abstraction for controlling a LED.
`class ` [`opcua::LedManager`](#classopcua_1_1_led_manager) | Provides methods for creating and storing LED objects in a collection.
`class ` [`opcua::PwmOutput`](#classopcua_1_1_pwm_output) | Provides an OPC UA variable node abstraction for a PWM output.
`class ` [`opcua::PwmOutputManager`](#classopcua_1_1_pwm_output_manager) | Provides methods for creating and storing [PwmOutput](#classopcua_1_1_pwm_output) objects in a collection.
`class ` [`opcua::Relay`](#classopcua_1_1_relay) | Provides an OPC UA variable node abstraction for controlling a mechanical or solid-state relay.
`class ` [`opcua::RelayManager`](#classopcua_1_1_relay_manager) | Provides methods for creating and storing [Relay](#classopcua_1_1_relay) objects in a collection.
`class ` [`opcua::Opta`](#classopcua_1_1_opta) | Provides functions for exposing various IO capabilities of the Arduino [Opta](#classopcua_1_1_opta) via OPC UA.
`class ` [`opcua::OptaExpansionManager`](#classopcua_1_1_opta_expansion_manager) | High-level class to create OPC UA representations for Arduino [Opta](#classopcua_1_1_opta) digital and analog expansion boards.
`class ` [`opcua::OptaVariant`](#classopcua_1_1_opta_variant) | Enables determination of [Opta](#classopcua_1_1_opta) variant (WiFi, RS485, Lite) on which the OPC UA firmware is running on.
`class ` [`opcua::NTPUtils`](#classopcua_1_1_n_t_p_utils) | Provides a method for obtaining the current time via UDP.
`class ` [`opcua::ExpansionType`](#classopcua_1_1_expansion_type) | Provides a method for converting Arduino_Opta_Blueprint's ExpansionType_t to a human readable string.

## Members

#### `public time_t ` [`timeToStr`](#namespaceopcua_1a5eab7edc9a71d850d6b47ae50e214c08)`(char const * time)` <a id="namespaceopcua_1a5eab7edc9a71d850d6b47ae50e214c08" class="anchor"></a>

Converts a date/time string as generated by **DATE** into a time_t value. 
#### Parameters
* `time` current time as generated by **DATE**. 

#### Returns
String converted into Unix Time (seconds since Epoch)

# class `opcua::AnalogExpansion` <a id="classopcua_1_1_analog_expansion" class="anchor"></a>

```cpp
class opcua::AnalogExpansion
  : public opcua::Expansion
```

Provides an OPC UA object node abstraction for an Analog [Expansion](#classopcua_1_1_expansion) board.

This class creates an OPC UA object node which is representing an Arduino [Opta](#classopcua_1_1_opta) Analog [Expansion](#classopcua_1_1_expansion) board. Furthermore, this class provides methods for adding OPC UA exposure to all IO entities supported by the Analog [Expansion](#classopcua_1_1_expansion) board.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`AnalogExpansion`](#classopcua_1_1_analog_expansion_1a8922c1b320139757c7200ec568d25324) | Constructor of the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class. Note: Objects should not be instantiated directly, but by using [AnalogExpansion::create](#classopcua_1_1_analog_expansion_1a13a953b2d732d52f0c6772957b45abe8).  |
| [`~AnalogExpansion`](#classopcua_1_1_analog_expansion_1ad9a18c730a6451d543e48a91bf7eeff1) | Destructor of the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class. |
| [`toSKUString`](#classopcua_1_1_analog_expansion_1a37b9a4657d1e8c0cf2e1d0322adfa27c) | Returns the SKU number of the Analog [Expansion](#classopcua_1_1_expansion) board as std::string.  |
| [`addAnalogInput`](#classopcua_1_1_analog_expansion_1afcb118a0a514b13fbc01167bca6f118d) | This function creates a new [opcua::AnalogInput](#classopcua_1_1_analog_input) and adds it to the [opcua::AnalogInputManager](#classopcua_1_1_analog_input_manager) belonging to the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class.  |
| [`addAnalogOutput`](#classopcua_1_1_analog_expansion_1a5b671c357e09846ad1a74ef2df641545) | This function creates a new [opcua::AnalogOutput](#classopcua_1_1_analog_output) and adds it to the [opcua::AnalogOutputManager](#classopcua_1_1_analog_output_manager) belonging to the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class.  |
| [`addPwmOutput`](#classopcua_1_1_analog_expansion_1ab0f7267b0b2eb714ce87e234df6f2985) | This function creates a new [opcua::PwmOutput](#classopcua_1_1_pwm_output) and adds it to the [opcua::PwmOutputManager](#classopcua_1_1_pwm_output_manager) belonging to the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class.  |
| [`addLedOutput`](#classopcua_1_1_analog_expansion_1ad7b7fdcc0c27bdd2671a1b90be26d38c) | This function creates a new opcua::LED and adds it to the opcua::LEDManager belonging to the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class.  |
| [`SharedPtr`](#classopcua_1_1_analog_expansion_1a3a99cadc0594891e0ce7ee9acc06d1f3) | SharedPtr is std::shared_ptr of an [AnalogExpansion](#classopcua_1_1_analog_expansion) class. |
| [`create`](#classopcua_1_1_analog_expansion_1a13a953b2d732d52f0c6772957b45abe8) | Creates a new instance of the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class, creating an OPC UA object node abstraction in the process.  |

## Members

### `AnalogExpansion` <a id="classopcua_1_1_analog_expansion_1a8922c1b320139757c7200ec568d25324" class="anchor"></a>

```cpp
AnalogExpansion(UA_Server * server, UA_NodeId const parent_node_id, char * display_name, char * node_name)
```

Constructor of the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class. Note: Objects should not be instantiated directly, but by using [AnalogExpansion::create](#classopcua_1_1_analog_expansion_1a13a953b2d732d52f0c6772957b45abe8). 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `display_name` Character string providing an easy identifiable name for the object node representing an analog expansion board, i.e. "Arduino Opta Expansion 1: Analog". 

* `node_name` The unique node name identifying the Analog [Expansion](#classopcua_1_1_expansion) Board object node within the OPC UA tree.
<hr />

### `~AnalogExpansion` <a id="classopcua_1_1_analog_expansion_1ad9a18c730a6451d543e48a91bf7eeff1" class="anchor"></a>

```cpp
virtual ~AnalogExpansion() = default
```

Destructor of the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class.
<hr />

### `toSKUString` <a id="classopcua_1_1_analog_expansion_1a37b9a4657d1e8c0cf2e1d0322adfa27c" class="anchor"></a>

```cpp
inline virtual std::string toSKUString() const
```

Returns the SKU number of the Analog [Expansion](#classopcua_1_1_expansion) board as std::string. 
#### Returns
SKU number of Analog [Expansion](#classopcua_1_1_expansion) board
<hr />

### `addAnalogInput` <a id="classopcua_1_1_analog_expansion_1afcb118a0a514b13fbc01167bca6f118d" class="anchor"></a>

```cpp
void addAnalogInput(UA_Server * server, const char * display_name, AnalogInput::OnReadRequestFunc const on_read_request)
```

This function creates a new [opcua::AnalogInput](#classopcua_1_1_analog_input) and adds it to the [opcua::AnalogInputManager](#classopcua_1_1_analog_input_manager) belonging to the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog input, i.e. "Analog Input I1". 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the analog input.
<hr />

### `addAnalogOutput` <a id="classopcua_1_1_analog_expansion_1a5b671c357e09846ad1a74ef2df641545" class="anchor"></a>

```cpp
void addAnalogOutput(UA_Server * server, const char * display_name, AnalogOutput::OnReadRequestFunc const on_read_request, AnalogOutput::OnWriteRequestFunc const on_write_request)
```

This function creates a new [opcua::AnalogOutput](#classopcua_1_1_analog_output) and adds it to the [opcua::AnalogOutputManager](#classopcua_1_1_analog_output_manager) belonging to the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog output, i.e. "Analog Output O1". 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the analog output. 

* `on_write_request` Function pointer which is called during a write-access on the variable node representing the analog output.
<hr />

### `addPwmOutput` <a id="classopcua_1_1_analog_expansion_1ab0f7267b0b2eb714ce87e234df6f2985" class="anchor"></a>

```cpp
void addPwmOutput(UA_Server * server, const char * display_name, PwmOutput::SetPwmFunc const set_pwm_func, PwmOutput::GetPwmPeriodFunc const get_pwm_period_func, PwmOutput::GetPwmPulseWidthFunc const get_pwm_pulse_width_func)
```

This function creates a new [opcua::PwmOutput](#classopcua_1_1_pwm_output) and adds it to the [opcua::PwmOutputManager](#classopcua_1_1_pwm_output_manager) belonging to the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog output, i.e. "PWM Output P1". 

* `set_pwm_func` Function pointer which is called during a write-access on the variable node representing the PWM output and used for setting the PWM period and pulse width. 

* `get_pwm_period_func` Function pointer which is called during a read-access on the variable node representing the PWM output's period. 

* `get_pwm_pulse_width_func` Function pointer which is called during a read-access on the variable node representing the PWM output's pulse width.
<hr />

### `addLedOutput` <a id="classopcua_1_1_analog_expansion_1ad7b7fdcc0c27bdd2671a1b90be26d38c" class="anchor"></a>

```cpp
void addLedOutput(UA_Server * server, const char * display_name, Led::OnSetLedStateFunc const on_set_led_state)
```

This function creates a new opcua::LED and adds it to the opcua::LEDManager belonging to the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing a LED, i.e. "LED1". 

* `on_set_led_state` Function pointer which is called during a write-access on the variable node representing the LED.
<hr />

### `SharedPtr` <a id="classopcua_1_1_analog_expansion_1a3a99cadc0594891e0ce7ee9acc06d1f3" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [AnalogExpansion](#classopcua_1_1_analog_expansion) class.
<hr />

### `create` <a id="classopcua_1_1_analog_expansion_1a13a953b2d732d52f0c6772957b45abe8" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const parent_node_id, uint8_t const exp_num)
```

Creates a new instance of the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class, creating an OPC UA object node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `exp_num` A numerical identifier provided by the Arduino_Opta_Blueprint library and identifying the number of the expansion module in the daisy-chain of expansion modules, i.e. exp_num = 2 refers to the second connect expansion module. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion).
<hr />

# class `opcua::DigitalExpansion` <a id="classopcua_1_1_digital_expansion" class="anchor"></a>

```cpp
class opcua::DigitalExpansion
  : public opcua::Expansion
```

Provides an OPC UA object node abstraction for both Digital [Expansion](#classopcua_1_1_expansion) board.

This class creates an OPC UA object node which is representing an Arduino [Opta](#classopcua_1_1_opta) Digital [Expansion](#classopcua_1_1_expansion) board. Furthermore, this class provides methods for adding OPC UA exposure to all IO entities supported by the Digital [Expansion](#classopcua_1_1_expansion) board.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`DigitalExpansion`](#classopcua_1_1_digital_expansion_1a83aace406ca80bde86e563dcf4ea12fe) | Constructor of the [opcua::DigitalExpansion](#classopcua_1_1_digital_expansion) class.  |
| [`~DigitalExpansion`](#classopcua_1_1_digital_expansion_1ab446f1129440aa88fe7d3fd133fc8bb1) | Destructor of the [opcua::DigitalExpansion](#classopcua_1_1_digital_expansion) class. |
| [`addAnalogInput`](#classopcua_1_1_digital_expansion_1a501296ef972bfe1a802af37ecdae0cc3) | This function creates a new [opcua::AnalogInput](#classopcua_1_1_analog_input) and adds it to the [opcua::AnalogInputManager](#classopcua_1_1_analog_input_manager) belonging to the [opcua::DigitalExpansion](#classopcua_1_1_digital_expansion) class.  |
| [`addDigitalInput`](#classopcua_1_1_digital_expansion_1a652a659e22a40ebf9bc371efc8b97475) | This function creates a new [opcua::DigitalInput](#classopcua_1_1_digital_input) and adds it to the [opcua::DigitalInputManager](#classopcua_1_1_digital_input_manager) belonging to the [opcua::DigitalExpansion](#classopcua_1_1_digital_expansion) class.  |
| [`addRelayOutput`](#classopcua_1_1_digital_expansion_1a05f43725e0d6b0f3b613e44cd340bd7d) | This function creates a new [opcua::Relay](#classopcua_1_1_relay) and adds it to the [opcua::RelayManager](#classopcua_1_1_relay_manager) belonging to the [opcua::DigitalExpansion](#classopcua_1_1_digital_expansion) class.  |
| [`SharedPtr`](#classopcua_1_1_digital_expansion_1a808388135d0335260aed3725a4ca8eb9) | SharedPtr is std::shared_ptr of an [DigitalExpansion](#classopcua_1_1_digital_expansion) class. |

## Members

### `DigitalExpansion` <a id="classopcua_1_1_digital_expansion_1a83aace406ca80bde86e563dcf4ea12fe" class="anchor"></a>

```cpp
DigitalExpansion(UA_Server * server, UA_NodeId const parent_node_id, char * display_name, char * node_name, char * model_name)
```

Constructor of the [opcua::DigitalExpansion](#classopcua_1_1_digital_expansion) class. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `display_name` Character string providing an easy identifiable name for the object node representing an analog expansion board, i.e. "Arduino Opta Expansion 1: Analog". 

* `node_name` The unique node name identifying the Analog [Expansion](#classopcua_1_1_expansion) Board object node within the OPC UA tree. 

* `model_name` This parameter must be filled with the SKU of the actual Digital [Expansion](#classopcua_1_1_expansion) board.
<hr />

### `~DigitalExpansion` <a id="classopcua_1_1_digital_expansion_1ab446f1129440aa88fe7d3fd133fc8bb1" class="anchor"></a>

```cpp
virtual ~DigitalExpansion() = default
```

Destructor of the [opcua::DigitalExpansion](#classopcua_1_1_digital_expansion) class.
<hr />

### `addAnalogInput` <a id="classopcua_1_1_digital_expansion_1a501296ef972bfe1a802af37ecdae0cc3" class="anchor"></a>

```cpp
void addAnalogInput(UA_Server * server, const char * display_name, AnalogInput::OnReadRequestFunc const on_read_request)
```

This function creates a new [opcua::AnalogInput](#classopcua_1_1_analog_input) and adds it to the [opcua::AnalogInputManager](#classopcua_1_1_analog_input_manager) belonging to the [opcua::DigitalExpansion](#classopcua_1_1_digital_expansion) class. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog input, i.e. "Analog Input I1". 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the analog input.
<hr />

### `addDigitalInput` <a id="classopcua_1_1_digital_expansion_1a652a659e22a40ebf9bc371efc8b97475" class="anchor"></a>

```cpp
void addDigitalInput(UA_Server * server, const char * display_name, DigitalInput::OnReadRequestFunc const on_read_request)
```

This function creates a new [opcua::DigitalInput](#classopcua_1_1_digital_input) and adds it to the [opcua::DigitalInputManager](#classopcua_1_1_digital_input_manager) belonging to the [opcua::DigitalExpansion](#classopcua_1_1_digital_expansion) class. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog input, i.e. "Digital Input I1". 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the digital input.
<hr />

### `addRelayOutput` <a id="classopcua_1_1_digital_expansion_1a05f43725e0d6b0f3b613e44cd340bd7d" class="anchor"></a>

```cpp
void addRelayOutput(UA_Server * server, const char * display_name, Relay::OnSetRelayStateFunc const on_set_relay_state)
```

This function creates a new [opcua::Relay](#classopcua_1_1_relay) and adds it to the [opcua::RelayManager](#classopcua_1_1_relay_manager) belonging to the [opcua::DigitalExpansion](#classopcua_1_1_digital_expansion) class. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing a relay, i.e. "Relay 1". 

* `on_set_relay_state` Function pointer which is called during a write-access on the variable node representing a relay.
<hr />

### `SharedPtr` <a id="classopcua_1_1_digital_expansion_1a808388135d0335260aed3725a4ca8eb9" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [DigitalExpansion](#classopcua_1_1_digital_expansion) class.
<hr />

# class `opcua::DigitalMechExpansion` <a id="classopcua_1_1_digital_mech_expansion" class="anchor"></a>

```cpp
class opcua::DigitalMechExpansion
  : public opcua::DigitalExpansion
```

Provides an OPC UA object node abstraction for a Digital [Expansion](#classopcua_1_1_expansion) board with mechanical relays.

This class creates an OPC UA object node which is representing an Arduino [Opta](#classopcua_1_1_opta) Digital [Expansion](#classopcua_1_1_expansion) board with mechanical relays.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`DigitalMechExpansion`](#classopcua_1_1_digital_mech_expansion_1ad7349ea0108d988330c6b2623ef99696) | Constructor of the [opcua::DigitalMechExpansion](#classopcua_1_1_digital_mech_expansion) class. Note: Objects should not be instantiated directly, but by using [DigitalMechExpansion::create](#classopcua_1_1_digital_mech_expansion_1af777877670455dbd9032ca0aa47fcee3).  |
| [`~DigitalMechExpansion`](#classopcua_1_1_digital_mech_expansion_1a10367a637954294193c4391c7507a8bc) | Destructor of the [opcua::DigitalMechExpansion](#classopcua_1_1_digital_mech_expansion) class. |
| [`toSKUString`](#classopcua_1_1_digital_mech_expansion_1a5d81cd280f0ab93fd5380d070c3d48b6) | Returns the SKU number of the Digital [Expansion](#classopcua_1_1_expansion) board with mechanical relays as std::string.  |
| [`SharedPtr`](#classopcua_1_1_digital_mech_expansion_1abc60f0e66b8f12e051497c58b113a16e) | SharedPtr is std::shared_ptr of an [DigitalMechExpansion](#classopcua_1_1_digital_mech_expansion) class. |
| [`create`](#classopcua_1_1_digital_mech_expansion_1af777877670455dbd9032ca0aa47fcee3) | Creates a new instance of the [opcua::DigitalMechExpansion](#classopcua_1_1_digital_mech_expansion) class, creating an OPC UA object node abstraction in the process.  |

## Members

### `DigitalMechExpansion` <a id="classopcua_1_1_digital_mech_expansion_1ad7349ea0108d988330c6b2623ef99696" class="anchor"></a>

```cpp
inline DigitalMechExpansion(UA_Server * server, UA_NodeId const parent_node_id, char * display_name, char * node_name)
```

Constructor of the [opcua::DigitalMechExpansion](#classopcua_1_1_digital_mech_expansion) class. Note: Objects should not be instantiated directly, but by using [DigitalMechExpansion::create](#classopcua_1_1_digital_mech_expansion_1af777877670455dbd9032ca0aa47fcee3). 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `display_name` Character string providing an easy identifiable name for the object node representing an analog expansion board, i.e. "Arduino Opta Expansion 1: Digital (Mechanical)". 

* `node_name` The unique node name identifying the Analog [Expansion](#classopcua_1_1_expansion) Board object node within the OPC UA tree.
<hr />

### `~DigitalMechExpansion` <a id="classopcua_1_1_digital_mech_expansion_1a10367a637954294193c4391c7507a8bc" class="anchor"></a>

```cpp
virtual ~DigitalMechExpansion() = default
```

Destructor of the [opcua::DigitalMechExpansion](#classopcua_1_1_digital_mech_expansion) class.
<hr />

### `toSKUString` <a id="classopcua_1_1_digital_mech_expansion_1a5d81cd280f0ab93fd5380d070c3d48b6" class="anchor"></a>

```cpp
inline virtual std::string toSKUString() const
```

Returns the SKU number of the Digital [Expansion](#classopcua_1_1_expansion) board with mechanical relays as std::string. 
#### Returns
SKU number of Digital [Expansion](#classopcua_1_1_expansion) board / mechanical relays
<hr />

### `SharedPtr` <a id="classopcua_1_1_digital_mech_expansion_1abc60f0e66b8f12e051497c58b113a16e" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [DigitalMechExpansion](#classopcua_1_1_digital_mech_expansion) class.
<hr />

### `create` <a id="classopcua_1_1_digital_mech_expansion_1af777877670455dbd9032ca0aa47fcee3" class="anchor"></a>

```cpp
inline static SharedPtr create(UA_Server * server, UA_NodeId const parent_node_id, uint8_t const exp_num)
```

Creates a new instance of the [opcua::DigitalMechExpansion](#classopcua_1_1_digital_mech_expansion) class, creating an OPC UA object node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `exp_num` A numerical identifier provided by the Arduino_Opta_Blueprint library and identifying the number of the expansion module in the daisy-chain of expansion modules, i.e. exp_num = 2 refers to the second connect expansion module. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::DigitalMechExpansion](#classopcua_1_1_digital_mech_expansion).
<hr />

# class `opcua::DigitalStSolidExpansion` <a id="classopcua_1_1_digital_st_solid_expansion" class="anchor"></a>

```cpp
class opcua::DigitalStSolidExpansion
  : public opcua::DigitalExpansion
```

Provides an OPC UA object node abstraction for a Digital [Expansion](#classopcua_1_1_expansion) board with solid-state relays.

This class creates an OPC UA object node which is representing an Arduino [Opta](#classopcua_1_1_opta) Digital [Expansion](#classopcua_1_1_expansion) board with solid-state relays.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`DigitalStSolidExpansion`](#classopcua_1_1_digital_st_solid_expansion_1a896bf61b18c63b055343d0b5f408dbb0) | Constructor of the [opcua::DigitalStSolidExpansion](#classopcua_1_1_digital_st_solid_expansion) class. Note: Objects should not be instantiated directly, but by using [DigitalStSolidExpansion::create](#classopcua_1_1_digital_st_solid_expansion_1a6b94afd184c4d435cea8ad32297fc045).  |
| [`~DigitalStSolidExpansion`](#classopcua_1_1_digital_st_solid_expansion_1afb8021cf2b6041d723c2a3c1888ed5f7) | Destructor of the [opcua::DigitalStSolidExpansion](#classopcua_1_1_digital_st_solid_expansion) class. |
| [`toSKUString`](#classopcua_1_1_digital_st_solid_expansion_1a67cb3b12dfc74b9747257daf1c481d99) | Returns the SKU number of the Digital [Expansion](#classopcua_1_1_expansion) board with solid-state relays as std::string.  |
| [`SharedPtr`](#classopcua_1_1_digital_st_solid_expansion_1a8bb44b6e51c634b1910dafb066f4e448) | SharedPtr is std::shared_ptr of an [DigitalStSolidExpansion](#classopcua_1_1_digital_st_solid_expansion) class. |
| [`create`](#classopcua_1_1_digital_st_solid_expansion_1a6b94afd184c4d435cea8ad32297fc045) | Creates a new instance of the [opcua::DigitalStSolidExpansion](#classopcua_1_1_digital_st_solid_expansion) class, creating an OPC UA object node abstraction in the process.  |

## Members

### `DigitalStSolidExpansion` <a id="classopcua_1_1_digital_st_solid_expansion_1a896bf61b18c63b055343d0b5f408dbb0" class="anchor"></a>

```cpp
inline DigitalStSolidExpansion(UA_Server * server, UA_NodeId const parent_node_id, char * display_name, char * node_name)
```

Constructor of the [opcua::DigitalStSolidExpansion](#classopcua_1_1_digital_st_solid_expansion) class. Note: Objects should not be instantiated directly, but by using [DigitalStSolidExpansion::create](#classopcua_1_1_digital_st_solid_expansion_1a6b94afd184c4d435cea8ad32297fc045). 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `display_name` Character string providing an easy identifiable name for the object node representing an analog expansion board, i.e. "Arduino Opta Expansion 1: Digital (Solid State)". 

* `node_name` The unique node name identifying the Analog [Expansion](#classopcua_1_1_expansion) Board object node within the OPC UA tree.
<hr />

### `~DigitalStSolidExpansion` <a id="classopcua_1_1_digital_st_solid_expansion_1afb8021cf2b6041d723c2a3c1888ed5f7" class="anchor"></a>

```cpp
virtual ~DigitalStSolidExpansion() = default
```

Destructor of the [opcua::DigitalStSolidExpansion](#classopcua_1_1_digital_st_solid_expansion) class.
<hr />

### `toSKUString` <a id="classopcua_1_1_digital_st_solid_expansion_1a67cb3b12dfc74b9747257daf1c481d99" class="anchor"></a>

```cpp
inline virtual std::string toSKUString() const
```

Returns the SKU number of the Digital [Expansion](#classopcua_1_1_expansion) board with solid-state relays as std::string. 
#### Returns
SKU number of Digital [Expansion](#classopcua_1_1_expansion) board / solid-state relays
<hr />

### `SharedPtr` <a id="classopcua_1_1_digital_st_solid_expansion_1a8bb44b6e51c634b1910dafb066f4e448" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [DigitalStSolidExpansion](#classopcua_1_1_digital_st_solid_expansion) class.
<hr />

### `create` <a id="classopcua_1_1_digital_st_solid_expansion_1a6b94afd184c4d435cea8ad32297fc045" class="anchor"></a>

```cpp
inline static SharedPtr create(UA_Server * server, UA_NodeId const parent_node_id, uint8_t const exp_num)
```

Creates a new instance of the [opcua::DigitalStSolidExpansion](#classopcua_1_1_digital_st_solid_expansion) class, creating an OPC UA object node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `exp_num` A numerical identifier provided by the Arduino_Opta_Blueprint library and identifying the number of the expansion module in the daisy-chain of expansion modules, i.e. exp_num = 2 refers to the second connect expansion module. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::DigitalStSolidExpansion](#classopcua_1_1_digital_st_solid_expansion).
<hr />

# class `opcua::Expansion` <a id="classopcua_1_1_expansion" class="anchor"></a>

Provides a generic base class for various expansion board specificic derived classes.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`Expansion`](#classopcua_1_1_expansion_1a52b0201d688fe43ddf8e662ce4a2ec32) | Constructor of the [opcua::Expansion](#classopcua_1_1_expansion) class.  |
| [`~Expansion`](#classopcua_1_1_expansion_1a10a1451bd459c4375e7dcc0c1ffabed7) | Destructor of the [opcua::Expansion](#classopcua_1_1_expansion) class. |
| [`toSKUString`](#classopcua_1_1_expansion_1ac2087b6c0d2153a8fd760fe5c7e689df) | Returns the SKU number of a concrete expansion board, must be overridden in derived classes.  |
| [`SharedPtr`](#classopcua_1_1_expansion_1a0e496feb6cd2259a2a9f4cded8f9076a) | SharedPtr is std::shared_ptr of an [Expansion](#classopcua_1_1_expansion) class. |

## Members

### `Expansion` <a id="classopcua_1_1_expansion_1a52b0201d688fe43ddf8e662ce4a2ec32" class="anchor"></a>

```cpp
Expansion(UA_Server * server, UA_NodeId const parent_node_id, char * display_name, char * node_name, char * model_name)
```

Constructor of the [opcua::Expansion](#classopcua_1_1_expansion) class. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `display_name` Character string providing an easy identifiable name for the object node representing an analog expansion board, i.e. "Arduino Opta Expansion 1: Analog". 

* `node_name` The unique node name identifying the Analog [Expansion](#classopcua_1_1_expansion) Board object node within the OPC UA tree. 

* `model_name` This parameter must be filled with the SKU of the actual expansion board.
<hr />

### `~Expansion` <a id="classopcua_1_1_expansion_1a10a1451bd459c4375e7dcc0c1ffabed7" class="anchor"></a>

```cpp
virtual ~Expansion() = default
```

Destructor of the [opcua::Expansion](#classopcua_1_1_expansion) class.
<hr />

### `toSKUString` <a id="classopcua_1_1_expansion_1ac2087b6c0d2153a8fd760fe5c7e689df" class="anchor"></a>

```cpp
std::string toSKUString() const
```

Returns the SKU number of a concrete expansion board, must be overridden in derived classes. 
#### Returns
SKU number of concrete expansion board.
<hr />

### `SharedPtr` <a id="classopcua_1_1_expansion_1a0e496feb6cd2259a2a9f4cded8f9076a" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [Expansion](#classopcua_1_1_expansion) class.
<hr />

# class `opcua::AnalogInput` <a id="classopcua_1_1_analog_input" class="anchor"></a>

Provides an OPC UA variable node abstraction for an analog input.

This class creates an OPC UA variable node which is representing an analog input. When reading from the OPC UA variable node a callback is triggered which performs the actual analog readout of an analog input pin.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`AnalogInput`](#classopcua_1_1_analog_input_1ae30d8e20ce45b11730cdb43c4f9c16c4) | Constructor of the [opcua::AnalogInput](#classopcua_1_1_analog_input) class. Note: Objects should not be instantiated directly, but by using [AnalogInput::create](#classopcua_1_1_analog_input_1a11a3e1f0bdef544ba77c15c16d5a2689).  |
| [`onReadRequest`](#classopcua_1_1_analog_input_1a3bacb7c68ba84388ad5c2f88012affe0) | This function is called by the framework when a read-access on the variable node representing the analog input is performed. Note: This function should not be called directly, it will be invoked by the framework.  |
| [`SharedPtr`](#classopcua_1_1_analog_input_1abfac39c8c3cfda600c49522bb1b433c0) | SharedPtr is std::shared_ptr of an [AnalogInput](#classopcua_1_1_analog_input) class. |
| [`OnReadRequestFunc`](#classopcua_1_1_analog_input_1a76301ddd02080ac8f8e1916b33d31116) | OnReadRequestFunc is definition for a callback which is called during a read-access on the variable node representing the analog input performing the actual analog reading. |
| [`create`](#classopcua_1_1_analog_input_1a11a3e1f0bdef544ba77c15c16d5a2689) | Creates a new instance of the [opcua::AnalogInput](#classopcua_1_1_analog_input) class, creating an OPC UA variable node abstraction in the process.  |

## Members

### `AnalogInput` <a id="classopcua_1_1_analog_input_1ae30d8e20ce45b11730cdb43c4f9c16c4" class="anchor"></a>

```cpp
AnalogInput(UA_NodeId const & node_id, OnReadRequestFunc const on_read_request)
```

Constructor of the [opcua::AnalogInput](#classopcua_1_1_analog_input) class. Note: Objects should not be instantiated directly, but by using [AnalogInput::create](#classopcua_1_1_analog_input_1a11a3e1f0bdef544ba77c15c16d5a2689). 
#### Parameters
* `node_id` OPC UA node id of the variable node representing the analog input. 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the analog input.
<hr />

### `onReadRequest` <a id="classopcua_1_1_analog_input_1a3bacb7c68ba84388ad5c2f88012affe0" class="anchor"></a>

```cpp
void onReadRequest(UA_Server * server, UA_NodeId const * node_id)
```

This function is called by the framework when a read-access on the variable node representing the analog input is performed. Note: This function should not be called directly, it will be invoked by the framework. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `node_id` OPC UA node id of the variable node representing the analog input.
<hr />

### `SharedPtr` <a id="classopcua_1_1_analog_input_1abfac39c8c3cfda600c49522bb1b433c0" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [AnalogInput](#classopcua_1_1_analog_input) class.
<hr />

### `OnReadRequestFunc` <a id="classopcua_1_1_analog_input_1a76301ddd02080ac8f8e1916b33d31116" class="anchor"></a>

```cpp
typedef OnReadRequestFunc
```

OnReadRequestFunc is definition for a callback which is called during a read-access on the variable node representing the analog input performing the actual analog reading.
<hr />

### `create` <a id="classopcua_1_1_analog_input_1a11a3e1f0bdef544ba77c15c16d5a2689" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const & parent_node_id, const char * display_name, OnReadRequestFunc const on_read_request)
```

Creates a new instance of the [opcua::AnalogInput](#classopcua_1_1_analog_input) class, creating an OPC UA variable node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog input, i.e. "Analog Input I1". 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the analog input. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::AnalogInput](#classopcua_1_1_analog_input).
<hr />

# class `opcua::AnalogInputManager` <a id="classopcua_1_1_analog_input_manager" class="anchor"></a>

Provides methods for creating and storing [AnalogInput](#classopcua_1_1_analog_input) objects in a collection.

This class creates an OPC UA object node which is representing all analog inputs of a given OPC UA devices. Analog inputs can be created and added to the OPC UA object node using this class.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`AnalogInputManager`](#classopcua_1_1_analog_input_manager_1a4acded183249c128a0577d592434667a) | Constructor of the [opcua::AnalogInputManager](#classopcua_1_1_analog_input_manager) class. Note: Objects should not be instantiated directly, but by using [AnalogInputManager::create](#classopcua_1_1_analog_input_manager_1a68d99b738809b67b102ce91fbfbfaf4c).  |
| [`addAnalogInput`](#classopcua_1_1_analog_input_manager_1a988569b7fbfc49f7288e044e60e19d41) | This function creates a new [opcua::AnalogInput](#classopcua_1_1_analog_input) and adds it to the [opcua::AnalogInputManager](#classopcua_1_1_analog_input_manager).  |
| [`SharedPtr`](#classopcua_1_1_analog_input_manager_1aaf63f82d26fae9a89478d568acdbd390) | SharedPtr is std::shared_ptr of an [AnalogInputManager](#classopcua_1_1_analog_input_manager) class. |
| [`create`](#classopcua_1_1_analog_input_manager_1a68d99b738809b67b102ce91fbfbfaf4c) | Creates a new instance of the [opcua::AnalogInputManager](#classopcua_1_1_analog_input_manager) class, creating an OPC UA object node abstraction in the process.  |

## Members

### `AnalogInputManager` <a id="classopcua_1_1_analog_input_manager_1a4acded183249c128a0577d592434667a" class="anchor"></a>

```cpp
AnalogInputManager(UA_NodeId const & node_id)
```

Constructor of the [opcua::AnalogInputManager](#classopcua_1_1_analog_input_manager) class. Note: Objects should not be instantiated directly, but by using [AnalogInputManager::create](#classopcua_1_1_analog_input_manager_1a68d99b738809b67b102ce91fbfbfaf4c). 
#### Parameters
* `node_id` OPC UA node id of the object node representing all analog inputs.
<hr />

### `addAnalogInput` <a id="classopcua_1_1_analog_input_manager_1a988569b7fbfc49f7288e044e60e19d41" class="anchor"></a>

```cpp
void addAnalogInput(UA_Server * server, const char * display_name, AnalogInput::OnReadRequestFunc const on_read_request)
```

This function creates a new [opcua::AnalogInput](#classopcua_1_1_analog_input) and adds it to the [opcua::AnalogInputManager](#classopcua_1_1_analog_input_manager). 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog input, i.e. "Analog Input I1". 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the analog input.
<hr />

### `SharedPtr` <a id="classopcua_1_1_analog_input_manager_1aaf63f82d26fae9a89478d568acdbd390" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [AnalogInputManager](#classopcua_1_1_analog_input_manager) class.
<hr />

### `create` <a id="classopcua_1_1_analog_input_manager_1a68d99b738809b67b102ce91fbfbfaf4c" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const parent_node_id)
```

Creates a new instance of the [opcua::AnalogInputManager](#classopcua_1_1_analog_input_manager) class, creating an OPC UA object node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::AnalogInputManager](#classopcua_1_1_analog_input_manager).
<hr />

# class `opcua::AnalogOutput` <a id="classopcua_1_1_analog_output" class="anchor"></a>

Provides an OPC UA variable node abstraction for an analog output.

This class creates an OPC UA variable node which is representing an analog output. When writing to the OPC UA variable node a callback is triggered which performs the setting of the output value of an actual analog output pin. When reading from the OPC UA variable node a different callback is triggered which performs the actual analog readout of an analog output pin.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`AnalogOutput`](#classopcua_1_1_analog_output_1a16ac9355b42c1d58d132babc5a207d2e) | Constructor of the [opcua::AnalogOutput](#classopcua_1_1_analog_output) class. Note: Objects should not be instantiated directly, but by using [AnalogOutput::create](#classopcua_1_1_analog_output_1a16351e621798694bb3e09bd84eaf5571).  |
| [`onReadRequest`](#classopcua_1_1_analog_output_1a277345f3fe918b57f3c9e10e80a6b29b) | This function is called by the framework when a read-access on the variable node representing the analog output is performed. Note: This function should not be called directly, it will be invoked by the framework.  |
| [`onWriteRequest`](#classopcua_1_1_analog_output_1a072e18e88dd85d16e949434951c9942c) | This function is called by the framework when a write-access on the variable node representing the analog output is performed. Note: This function should not be called directly, it will be invoked by the framework.  |
| [`SharedPtr`](#classopcua_1_1_analog_output_1aa104e63edd1f282690fb7853ac24f3dd) | SharedPtr is std::shared_ptr of an [AnalogOutput](#classopcua_1_1_analog_output) class. |
| [`OnReadRequestFunc`](#classopcua_1_1_analog_output_1a4763f65ceb33a44a12e1a24fb48a2e37) | OnReadRequestFunc is definition for a callback which is called during a read-access on the variable node representing the analog output performing the actual analog read-back from the current value of the analog output pin. |
| [`OnWriteRequestFunc`](#classopcua_1_1_analog_output_1a607044dc4f4a50e50f68a343c6bb4d1f) | OnWriteRequestFunc is definition for a callback which is called during a write-access on the variable node representing the analog output performing the actual writing of the desired output voltage to the analog output pin. |
| [`create`](#classopcua_1_1_analog_output_1a16351e621798694bb3e09bd84eaf5571) | Creates a new instance of the [opcua::AnalogOutput](#classopcua_1_1_analog_output) class, creating an OPC UA variable node abstraction in the process.  |

## Members

### `AnalogOutput` <a id="classopcua_1_1_analog_output_1a16ac9355b42c1d58d132babc5a207d2e" class="anchor"></a>

```cpp
AnalogOutput(UA_NodeId const & node_id, OnReadRequestFunc const on_read_request, OnWriteRequestFunc const on_write_request)
```

Constructor of the [opcua::AnalogOutput](#classopcua_1_1_analog_output) class. Note: Objects should not be instantiated directly, but by using [AnalogOutput::create](#classopcua_1_1_analog_output_1a16351e621798694bb3e09bd84eaf5571). 
#### Parameters
* `node_id` OPC UA node id of the variable node representing the analog output. 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the analog output. 

* `on_write_request` Function pointer which is called during a write-access on the variable node representing the analog output.
<hr />

### `onReadRequest` <a id="classopcua_1_1_analog_output_1a277345f3fe918b57f3c9e10e80a6b29b" class="anchor"></a>

```cpp
void onReadRequest(UA_Server * server, UA_NodeId const * node_id)
```

This function is called by the framework when a read-access on the variable node representing the analog output is performed. Note: This function should not be called directly, it will be invoked by the framework. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `node_id` OPC UA node id of the variable node representing the analog output.
<hr />

### `onWriteRequest` <a id="classopcua_1_1_analog_output_1a072e18e88dd85d16e949434951c9942c" class="anchor"></a>

```cpp
void onWriteRequest(UA_Server * server, UA_NodeId const * node_id, float const voltage)
```

This function is called by the framework when a write-access on the variable node representing the analog output is performed. Note: This function should not be called directly, it will be invoked by the framework. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `node_id` OPC UA node id of the variable node representing the analog output. 

* `voltage` Desired output voltage value for the analog output.
<hr />

### `SharedPtr` <a id="classopcua_1_1_analog_output_1aa104e63edd1f282690fb7853ac24f3dd" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [AnalogOutput](#classopcua_1_1_analog_output) class.
<hr />

### `OnReadRequestFunc` <a id="classopcua_1_1_analog_output_1a4763f65ceb33a44a12e1a24fb48a2e37" class="anchor"></a>

```cpp
typedef OnReadRequestFunc
```

OnReadRequestFunc is definition for a callback which is called during a read-access on the variable node representing the analog output performing the actual analog read-back from the current value of the analog output pin.
<hr />

### `OnWriteRequestFunc` <a id="classopcua_1_1_analog_output_1a607044dc4f4a50e50f68a343c6bb4d1f" class="anchor"></a>

```cpp
typedef OnWriteRequestFunc
```

OnWriteRequestFunc is definition for a callback which is called during a write-access on the variable node representing the analog output performing the actual writing of the desired output voltage to the analog output pin.
<hr />

### `create` <a id="classopcua_1_1_analog_output_1a16351e621798694bb3e09bd84eaf5571" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const & parent_node_id, const char * display_name, OnReadRequestFunc const on_read_request, OnWriteRequestFunc const on_write_request)
```

Creates a new instance of the [opcua::AnalogOutput](#classopcua_1_1_analog_output) class, creating an OPC UA variable node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog output, i.e. "Analog Output O1". 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the analog output. 

* `on_write_request` Function pointer which is called during a write-access on the variable node representing the analog output. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::AnalogOutput](#classopcua_1_1_analog_output).
<hr />

# class `opcua::AnalogOutputManager` <a id="classopcua_1_1_analog_output_manager" class="anchor"></a>

Provides methods for creating and storing [AnalogOutput](#classopcua_1_1_analog_output) objects in a collection.

This class creates an OPC UA object node which is representing all analog outputs of a given OPC UA devices. Analog outputs can be created and added to the OPC UA object node using this class.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`AnalogOutputManager`](#classopcua_1_1_analog_output_manager_1af41a1bda2ad81560a6e9d4c005df0cff) | Constructor of the [opcua::AnalogOutputManager](#classopcua_1_1_analog_output_manager) class. Note: Objects should not be instantiated directly, but by using [AnalogOutputManager::create](#classopcua_1_1_analog_output_manager_1ae70361c7213c9ddb274b607eb0dda289).  |
| [`addAnalogOutput`](#classopcua_1_1_analog_output_manager_1a4b2c54ec74b7ebacad77119df4850ec9) | This function creates a new [opcua::AnalogOutput](#classopcua_1_1_analog_output) and adds it to the [opcua::AnalogOutputManager](#classopcua_1_1_analog_output_manager).  |
| [`SharedPtr`](#classopcua_1_1_analog_output_manager_1a019101cbb84c3458e510153db559be63) | SharedPtr is std::shared_ptr of an [AnalogOutputManager](#classopcua_1_1_analog_output_manager) class. |
| [`create`](#classopcua_1_1_analog_output_manager_1ae70361c7213c9ddb274b607eb0dda289) | Creates a new instance of the [opcua::AnalogOutputManager](#classopcua_1_1_analog_output_manager) class, creating an OPC UA object node abstraction in the process.  |

## Members

### `AnalogOutputManager` <a id="classopcua_1_1_analog_output_manager_1af41a1bda2ad81560a6e9d4c005df0cff" class="anchor"></a>

```cpp
AnalogOutputManager(UA_NodeId const & node_id)
```

Constructor of the [opcua::AnalogOutputManager](#classopcua_1_1_analog_output_manager) class. Note: Objects should not be instantiated directly, but by using [AnalogOutputManager::create](#classopcua_1_1_analog_output_manager_1ae70361c7213c9ddb274b607eb0dda289). 
#### Parameters
* `node_id` OPC UA node id of the object node representing all analog outputs.
<hr />

### `addAnalogOutput` <a id="classopcua_1_1_analog_output_manager_1a4b2c54ec74b7ebacad77119df4850ec9" class="anchor"></a>

```cpp
void addAnalogOutput(UA_Server * server, const char * display_name, AnalogOutput::OnReadRequestFunc const on_read_request, AnalogOutput::OnWriteRequestFunc const on_write_request)
```

This function creates a new [opcua::AnalogOutput](#classopcua_1_1_analog_output) and adds it to the [opcua::AnalogOutputManager](#classopcua_1_1_analog_output_manager). 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog output, i.e. "Analog Output O1". 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the analog output. 

* `on_write_request` Function pointer which is called during a write-access on the variable node representing the analog output.
<hr />

### `SharedPtr` <a id="classopcua_1_1_analog_output_manager_1a019101cbb84c3458e510153db559be63" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [AnalogOutputManager](#classopcua_1_1_analog_output_manager) class.
<hr />

### `create` <a id="classopcua_1_1_analog_output_manager_1ae70361c7213c9ddb274b607eb0dda289" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const parent_node_id)
```

Creates a new instance of the [opcua::AnalogOutputManager](#classopcua_1_1_analog_output_manager) class, creating an OPC UA object node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::AnalogOutputManager](#classopcua_1_1_analog_output_manager).
<hr />

# class `opcua::UserButton` <a id="classopcua_1_1_user_button" class="anchor"></a>

Provides an OPC UA variable node abstraction for a user button input.

This class creates an OPC UA variable node which is representing a user button input. When reading from the OPC UA variable node a callback is triggered which performs the actual readout of the current state of the user button.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`UserButton`](#classopcua_1_1_user_button_1a8a18ad7cb6edfefc58a8f1fe7cd5b87d) | Constructor of the [opcua::UserButton](#classopcua_1_1_user_button) class. Note: Objects should not be instantiated directly, but by using [UserButton::create](#classopcua_1_1_user_button_1aaccd71565a95f2c50e33c52077c16171).  |
| [`onReadRequest`](#classopcua_1_1_user_button_1a211f565d05611888ec518de01ed6f120) | This function is called by the framework when a read-access on the variable node representing the user button input is performed. Note: This function should not be called directly, it will be invoked by the framework.  |
| [`SharedPtr`](#classopcua_1_1_user_button_1a5ae06baa98d9a035629c86076d96bfc5) | SharedPtr is std::shared_ptr of an [UserButton](#classopcua_1_1_user_button) class. |
| [`create`](#classopcua_1_1_user_button_1aaccd71565a95f2c50e33c52077c16171) | Creates a new instance of the [opcua::UserButton](#classopcua_1_1_user_button) class, creating an OPC UA variable node "User Button" abstraction in the process.  |

## Members

### `UserButton` <a id="classopcua_1_1_user_button_1a8a18ad7cb6edfefc58a8f1fe7cd5b87d" class="anchor"></a>

```cpp
UserButton(UA_NodeId const & node_id)
```

Constructor of the [opcua::UserButton](#classopcua_1_1_user_button) class. Note: Objects should not be instantiated directly, but by using [UserButton::create](#classopcua_1_1_user_button_1aaccd71565a95f2c50e33c52077c16171). 
#### Parameters
* `node_id` OPC UA node id of the variable node representing the user button.
<hr />

### `onReadRequest` <a id="classopcua_1_1_user_button_1a211f565d05611888ec518de01ed6f120" class="anchor"></a>

```cpp
void onReadRequest(UA_Server * server, UA_NodeId const * node_id)
```

This function is called by the framework when a read-access on the variable node representing the user button input is performed. Note: This function should not be called directly, it will be invoked by the framework. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `node_id` OPC UA node id of the variable node representing the user button.
<hr />

### `SharedPtr` <a id="classopcua_1_1_user_button_1a5ae06baa98d9a035629c86076d96bfc5" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [UserButton](#classopcua_1_1_user_button) class.
<hr />

### `create` <a id="classopcua_1_1_user_button_1aaccd71565a95f2c50e33c52077c16171" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const & parent_node_id)
```

Creates a new instance of the [opcua::UserButton](#classopcua_1_1_user_button) class, creating an OPC UA variable node "User Button" abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::UserButton](#classopcua_1_1_user_button).
<hr />

# class `opcua::DigitalInput` <a id="classopcua_1_1_digital_input" class="anchor"></a>

Provides an OPC UA variable node abstraction for a digital input.

This class creates an OPC UA variable node which is representing a digital input. When reading from the OPC UA variable node a callback is triggered which performs the actual readout of a digital input pin.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`DigitalInput`](#classopcua_1_1_digital_input_1a0da78dff02b4ef7bd8038bab590413da) | Constructor of the [opcua::DigitalInput](#classopcua_1_1_digital_input) class. Note: Objects should not be instantiated directly, but by using [DigitalInput::create](#classopcua_1_1_digital_input_1a844932d3127b5adeedd8c7dc5c0f2feb).  |
| [`onReadRequest`](#classopcua_1_1_digital_input_1a07d7f2c66fd7a856d28852fb72015243) | This function is called by the framework when a read-access on the variable node representing the digital input is performed. Note: This function should not be called directly, it will be invoked by the framework.  |
| [`SharedPtr`](#classopcua_1_1_digital_input_1a5ed8dab221203259b007d15bb332edc8) | SharedPtr is std::shared_ptr of a [DigitalInput](#classopcua_1_1_digital_input) class. |
| [`OnReadRequestFunc`](#classopcua_1_1_digital_input_1ae45c9b0e3cb2774b2e39431a7feccf41) | OnReadRequestFunc is definition for a callback which is called during a read-access on the variable node representing the digital input performing the actual digital reading. |
| [`create`](#classopcua_1_1_digital_input_1a844932d3127b5adeedd8c7dc5c0f2feb) | Creates a new instance of the [opcua::DigitalInput](#classopcua_1_1_digital_input) class, creating an OPC UA variable node abstraction in the process.  |

## Members

### `DigitalInput` <a id="classopcua_1_1_digital_input_1a0da78dff02b4ef7bd8038bab590413da" class="anchor"></a>

```cpp
DigitalInput(UA_NodeId const & node_id, OnReadRequestFunc const on_read_request)
```

Constructor of the [opcua::DigitalInput](#classopcua_1_1_digital_input) class. Note: Objects should not be instantiated directly, but by using [DigitalInput::create](#classopcua_1_1_digital_input_1a844932d3127b5adeedd8c7dc5c0f2feb). 
#### Parameters
* `node_id` OPC UA node id of the variable node representing the digital input. 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the digital input.
<hr />

### `onReadRequest` <a id="classopcua_1_1_digital_input_1a07d7f2c66fd7a856d28852fb72015243" class="anchor"></a>

```cpp
void onReadRequest(UA_Server * server, UA_NodeId const * node_id)
```

This function is called by the framework when a read-access on the variable node representing the digital input is performed. Note: This function should not be called directly, it will be invoked by the framework. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `node_id` OPC UA node id of the variable node representing the digital input.
<hr />

### `SharedPtr` <a id="classopcua_1_1_digital_input_1a5ed8dab221203259b007d15bb332edc8" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of a [DigitalInput](#classopcua_1_1_digital_input) class.
<hr />

### `OnReadRequestFunc` <a id="classopcua_1_1_digital_input_1ae45c9b0e3cb2774b2e39431a7feccf41" class="anchor"></a>

```cpp
typedef OnReadRequestFunc
```

OnReadRequestFunc is definition for a callback which is called during a read-access on the variable node representing the digital input performing the actual digital reading.
<hr />

### `create` <a id="classopcua_1_1_digital_input_1a844932d3127b5adeedd8c7dc5c0f2feb" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const & parent_node_id, const char * display_name, OnReadRequestFunc const on_read_request)
```

Creates a new instance of the [opcua::DigitalInput](#classopcua_1_1_digital_input) class, creating an OPC UA variable node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog input, i.e. "Digital Input I1". 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the digital input. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::DigitalInput](#classopcua_1_1_digital_input).
<hr />

# class `opcua::DigitalInputManager` <a id="classopcua_1_1_digital_input_manager" class="anchor"></a>

Provides methods for creating and storing [DigitalInput](#classopcua_1_1_digital_input) objects in a collection.

This class creates an OPC UA object node which is representing all digital inputs of a given OPC UA devices. Digital inputs can be created and added to the OPC UA object node using this class.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`DigitalInputManager`](#classopcua_1_1_digital_input_manager_1aa1a41ef9de849beb644a197c5d69c25e) | Constructor of the [opcua::DigitalInputManager](#classopcua_1_1_digital_input_manager) class. Note: Objects should not be instantiated directly, but by using [DigitalInputManager::create](#classopcua_1_1_digital_input_manager_1aa4f8ec8ba017a9c51d5e8ce14f0d5775).  |
| [`addDigitalInput`](#classopcua_1_1_digital_input_manager_1a75a40530489221d19a84ea88f63ae674) | This function creates a new [opcua::DigitalInput](#classopcua_1_1_digital_input) and adds it to the [opcua::DigitalInputManager](#classopcua_1_1_digital_input_manager).  |
| [`SharedPtr`](#classopcua_1_1_digital_input_manager_1a65c34fbf328ac93e94931922547e2bbf) | SharedPtr is std::shared_ptr of an [DigitalInputManager](#classopcua_1_1_digital_input_manager) class. |
| [`create`](#classopcua_1_1_digital_input_manager_1aa4f8ec8ba017a9c51d5e8ce14f0d5775) | Creates a new instance of the [opcua::DigitalInputManager](#classopcua_1_1_digital_input_manager) class, creating an OPC UA object node abstraction in the process.  |

## Members

### `DigitalInputManager` <a id="classopcua_1_1_digital_input_manager_1aa1a41ef9de849beb644a197c5d69c25e" class="anchor"></a>

```cpp
DigitalInputManager(UA_NodeId const & node_id)
```

Constructor of the [opcua::DigitalInputManager](#classopcua_1_1_digital_input_manager) class. Note: Objects should not be instantiated directly, but by using [DigitalInputManager::create](#classopcua_1_1_digital_input_manager_1aa4f8ec8ba017a9c51d5e8ce14f0d5775). 
#### Parameters
* `node_id` OPC UA node id of the object node representing all digital inputs.
<hr />

### `addDigitalInput` <a id="classopcua_1_1_digital_input_manager_1a75a40530489221d19a84ea88f63ae674" class="anchor"></a>

```cpp
void addDigitalInput(UA_Server * server, const char * display_name, DigitalInput::OnReadRequestFunc const on_read_request)
```

This function creates a new [opcua::DigitalInput](#classopcua_1_1_digital_input) and adds it to the [opcua::DigitalInputManager](#classopcua_1_1_digital_input_manager). 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog input, i.e. "Digital Input I1". 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the digital input.
<hr />

### `SharedPtr` <a id="classopcua_1_1_digital_input_manager_1a65c34fbf328ac93e94931922547e2bbf" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [DigitalInputManager](#classopcua_1_1_digital_input_manager) class.
<hr />

### `create` <a id="classopcua_1_1_digital_input_manager_1aa4f8ec8ba017a9c51d5e8ce14f0d5775" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const parent_node_id)
```

Creates a new instance of the [opcua::DigitalInputManager](#classopcua_1_1_digital_input_manager) class, creating an OPC UA object node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::DigitalInputManager](#classopcua_1_1_digital_input_manager).
<hr />

# class `opcua::Led` <a id="classopcua_1_1_led" class="anchor"></a>

Provides an OPC UA variable node abstraction for controlling a LED.

This class creates an OPC UA variable node which is representing a LED. When writing to the OPC UA variable node a callback is triggered which either turns the LED on or off.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`Led`](#classopcua_1_1_led_1a8fc018148c11c526eddb312b6765d647) | Constructor of the [opcua::Led](#classopcua_1_1_led) class. Note: Objects should not be instantiated directly, but by using [Led::create](#classopcua_1_1_led_1a629e48c791ae316f9ca0f08038ddcfe1).  |
| [`onWriteRequest`](#classopcua_1_1_led_1aaf8221b4bc407e27e19d4efdbc2eba4e) | This function is called by the framework when a write-access on the variable node representing a LED is performed. Note: This function should not be called directly, it will be invoked by the framework.  |
| [`SharedPtr`](#classopcua_1_1_led_1ae98b3b6f5b9fc5efa65f5cb9afd36d26) | SharedPtr is std::shared_ptr of a [Led](#classopcua_1_1_led) class. |
| [`OnSetLedStateFunc`](#classopcua_1_1_led_1aaf87efcccae9e4a8bdbf44f59aabb448) | OnSetLedStateFunc is definition for a callback which is called during a write-access on the variable node representing a LED subsequently turned the LED either ON (true) or OFF (false). |
| [`create`](#classopcua_1_1_led_1a629e48c791ae316f9ca0f08038ddcfe1) | Creates a new instance of the [opcua::Led](#classopcua_1_1_led) class, creating an OPC UA variable node abstraction in the process.  |

## Members

### `Led` <a id="classopcua_1_1_led_1a8fc018148c11c526eddb312b6765d647" class="anchor"></a>

```cpp
Led(UA_NodeId const & node_id, OnSetLedStateFunc const on_set_led_state)
```

Constructor of the [opcua::Led](#classopcua_1_1_led) class. Note: Objects should not be instantiated directly, but by using [Led::create](#classopcua_1_1_led_1a629e48c791ae316f9ca0f08038ddcfe1). 
#### Parameters
* `node_id` OPC UA node id of the variable node representing the analog output. 

* `on_set_led_state` Function pointer which is called during a write-access on the variable node representing the LED.
<hr />

### `onWriteRequest` <a id="classopcua_1_1_led_1aaf8221b4bc407e27e19d4efdbc2eba4e" class="anchor"></a>

```cpp
void onWriteRequest(UA_Server * server, UA_NodeId const * node_id, bool const value)
```

This function is called by the framework when a write-access on the variable node representing a LED is performed. Note: This function should not be called directly, it will be invoked by the framework. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `node_id` OPC UA node id of the variable node representing the LED. 

* `value` Desired state of the LED, true -> LED is turned ON, false -> LED is turned off.
<hr />

### `SharedPtr` <a id="classopcua_1_1_led_1ae98b3b6f5b9fc5efa65f5cb9afd36d26" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of a [Led](#classopcua_1_1_led) class.
<hr />

### `OnSetLedStateFunc` <a id="classopcua_1_1_led_1aaf87efcccae9e4a8bdbf44f59aabb448" class="anchor"></a>

```cpp
typedef OnSetLedStateFunc
```

OnSetLedStateFunc is definition for a callback which is called during a write-access on the variable node representing a LED subsequently turned the LED either ON (true) or OFF (false).
<hr />

### `create` <a id="classopcua_1_1_led_1a629e48c791ae316f9ca0f08038ddcfe1" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const & parent_node_id, const char * display_name, OnSetLedStateFunc const on_set_led_state)
```

Creates a new instance of the [opcua::Led](#classopcua_1_1_led) class, creating an OPC UA variable node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `display_name` Character string providing an easy identifiable name for the variable node representing a LED, i.e. "LED1". 

* `on_set_led_state` Function pointer which is called during a write-access on the variable node representing the LED. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::Led](#classopcua_1_1_led).
<hr />

# class `opcua::LedManager` <a id="classopcua_1_1_led_manager" class="anchor"></a>

Provides methods for creating and storing LED objects in a collection.

This class creates an OPC UA object node which is representing all LEDs of a given OPC UA devices. LEDs can be created and added to the OPC UA object node using this class.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`LedManager`](#classopcua_1_1_led_manager_1a1388f892d85f566acacca95b10b4c62a) | Constructor of the [opcua::LedManager](#classopcua_1_1_led_manager) class. Note: Objects should not be instantiated directly, but by using [LedManager::create](#classopcua_1_1_led_manager_1a34dfb842af941ecc7505050808885e0f).  |
| [`addLedOutput`](#classopcua_1_1_led_manager_1a70587aeb79078f2039790d595974bb3d) | This function creates a new opcua::LED and adds it to the opcua::LEDManager.  |
| [`SharedPtr`](#classopcua_1_1_led_manager_1add9494665b55716dcdb70b9ce1e0e1e2) | SharedPtr is std::shared_ptr of an [LedManager](#classopcua_1_1_led_manager) class. |
| [`create`](#classopcua_1_1_led_manager_1a34dfb842af941ecc7505050808885e0f) | Creates a new instance of the [opcua::LedManager](#classopcua_1_1_led_manager) class, creating an OPC UA object node abstraction in the process.  |

## Members

### `LedManager` <a id="classopcua_1_1_led_manager_1a1388f892d85f566acacca95b10b4c62a" class="anchor"></a>

```cpp
LedManager(UA_NodeId const & node_id)
```

Constructor of the [opcua::LedManager](#classopcua_1_1_led_manager) class. Note: Objects should not be instantiated directly, but by using [LedManager::create](#classopcua_1_1_led_manager_1a34dfb842af941ecc7505050808885e0f). 
#### Parameters
* `node_id` OPC UA node id of the object node representing all LEDs.
<hr />

### `addLedOutput` <a id="classopcua_1_1_led_manager_1a70587aeb79078f2039790d595974bb3d" class="anchor"></a>

```cpp
void addLedOutput(UA_Server * server, const char * display_name, Led::OnSetLedStateFunc const on_set_led_state)
```

This function creates a new opcua::LED and adds it to the opcua::LEDManager. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing a LED, i.e. "LED1". 

* `on_set_led_state` Function pointer which is called during a write-access on the variable node representing the LED.
<hr />

### `SharedPtr` <a id="classopcua_1_1_led_manager_1add9494665b55716dcdb70b9ce1e0e1e2" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [LedManager](#classopcua_1_1_led_manager) class.
<hr />

### `create` <a id="classopcua_1_1_led_manager_1a34dfb842af941ecc7505050808885e0f" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const parent_node_id)
```

Creates a new instance of the [opcua::LedManager](#classopcua_1_1_led_manager) class, creating an OPC UA object node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::LedManager](#classopcua_1_1_led_manager).
<hr />

# class `opcua::PwmOutput` <a id="classopcua_1_1_pwm_output" class="anchor"></a>

Provides an OPC UA variable node abstraction for a PWM output.

This class creates an OPC UA object node representing a PWM output with two OPC UA variable nodes representing the PWM output's period and pulse width. When writing to any OPC UA variable node a callback is triggered which performs the setting of the desired PWM period and pulse width. When reading from the OPC UA variable nodes different callback are triggered which performs reading back the actual PWM period and pulse width.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`PwmOutput`](#classopcua_1_1_pwm_output_1afb7531394d5f6949a1139efc8eb4a2d4) | Constructor of the [opcua::PwmOutput](#classopcua_1_1_pwm_output) class. Note: Objects should not be instantiated directly, but by using [PwmOutput::create](#classopcua_1_1_pwm_output_1a595c247e837f0e9a34e11b2b7b5a26ff).  |
| [`onReadRequestPwmPeriod`](#classopcua_1_1_pwm_output_1a996ade007bc33c1755cb7a58556d1b72) | This function is called by the framework when a read-access on the variable node representing the PWM output's period is performed. Note: This function should not be called directly, it will be invoked by the framework.  |
| [`onWriteRequestPwmPeriod`](#classopcua_1_1_pwm_output_1ab3361d34f09f0d478253c3eaeaa5d0b8) | This function is called by the framework when a write-access on the variable node representing the PWM output's period is performed. Note: This function should not be called directly, it will be invoked by the framework.  |
| [`onReadRequestPwmPulseWidth`](#classopcua_1_1_pwm_output_1ad94b10965197e6e34421e01826851e5b) | This function is called by the framework when a read-access on the variable node representing the PWM output's pulse width is performed. Note: This function should not be called directly, it will be invoked by the framework.  |
| [`onWriteRequestPwmPulseWidth`](#classopcua_1_1_pwm_output_1a9a88eb5f43679c66d0be1288158e055c) | This function is called by the framework when a write-access on the variable node representing the PWM output's pulse width is performed. Note: This function should not be called directly, it will be invoked by the framework.  |
| [`SharedPtr`](#classopcua_1_1_pwm_output_1ad871619fdeeccd340fad668e1d59c668) | SharedPtr is std::shared_ptr of a [PwmOutput](#classopcua_1_1_pwm_output) class. |
| [`SetPwmFunc`](#classopcua_1_1_pwm_output_1a979bc7c17dbac4dfb518a65dd92c1c56) | SetPwmFunc is definition for a callback which is called during a write-access on the variable nodes representing PWM period and PWM pulse width subsequently configuring the PWN output with the desired period and pulse width. |
| [`GetPwmPeriodFunc`](#classopcua_1_1_pwm_output_1ae9f4bd8023add25dda7eb038745e494c) | GetPwmPeriodFunc is definition for a callback which is called during a read-access on the variable node representing the PWM period and which is used to retrieve the actual PWM period. |
| [`GetPwmPulseWidthFunc`](#classopcua_1_1_pwm_output_1ad9ca89b1b476fac49dd9d32306b819af) | GetPwmPulseWidthFunc is definition for a callback which is called during a read-access on the variable node representing the PWM pulse width and which is used to retrieve the actual PWM pulse width. |
| [`create`](#classopcua_1_1_pwm_output_1a595c247e837f0e9a34e11b2b7b5a26ff) | Creates a new instance of the [opcua::PwmOutput](#classopcua_1_1_pwm_output) class, creating an OPC UA variable node abstraction in the process.  |

## Members

### `PwmOutput` <a id="classopcua_1_1_pwm_output_1afb7531394d5f6949a1139efc8eb4a2d4" class="anchor"></a>

```cpp
PwmOutput(UA_NodeId const & pwm_period_node_id, UA_NodeId const & pwm_pulse_width_node_id, SetPwmFunc const set_pwm_func, GetPwmPeriodFunc const get_pwm_period_func, GetPwmPulseWidthFunc const get_pwm_pulse_width_func)
```

Constructor of the [opcua::PwmOutput](#classopcua_1_1_pwm_output) class. Note: Objects should not be instantiated directly, but by using [PwmOutput::create](#classopcua_1_1_pwm_output_1a595c247e837f0e9a34e11b2b7b5a26ff). 
#### Parameters
* `pwm_period_node_id` OPC UA node id of the variable node representing the PWM output's period. 

* `pwm_pulse_width_node_id` OPC UA node id of the variable node representing the PWM output's pulse width. 

* `set_pwm_func` Function pointer which is called during a write-access on the variable node representing the PWM output and used for setting the PWM period and pulse width. 

* `get_pwm_period_func` Function pointer which is called during a read-access on the variable node representing the PWM output's period. 

* `get_pwm_pulse_width_func` Function pointer which is called during a read-access on the variable node representing the PWM output's pulse width.
<hr />

### `onReadRequestPwmPeriod` <a id="classopcua_1_1_pwm_output_1a996ade007bc33c1755cb7a58556d1b72" class="anchor"></a>

```cpp
void onReadRequestPwmPeriod(UA_Server * server, UA_NodeId const * node_id)
```

This function is called by the framework when a read-access on the variable node representing the PWM output's period is performed. Note: This function should not be called directly, it will be invoked by the framework. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `node_id` OPC UA node id of the variable node representing the PWM output's period.
<hr />

### `onWriteRequestPwmPeriod` <a id="classopcua_1_1_pwm_output_1ab3361d34f09f0d478253c3eaeaa5d0b8" class="anchor"></a>

```cpp
void onWriteRequestPwmPeriod(UA_Server * server, UA_NodeId const * pwm_period_node_id, uint32_t const pwm_period_us)
```

This function is called by the framework when a write-access on the variable node representing the PWM output's period is performed. Note: This function should not be called directly, it will be invoked by the framework. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `pwm_period_node_id` OPC UA node id of the variable node representing the PWM output's period. 

* `pwm_period_us` Desired PWM output period in microseconds.
<hr />

### `onReadRequestPwmPulseWidth` <a id="classopcua_1_1_pwm_output_1ad94b10965197e6e34421e01826851e5b" class="anchor"></a>

```cpp
void onReadRequestPwmPulseWidth(UA_Server * server, UA_NodeId const * node_id)
```

This function is called by the framework when a read-access on the variable node representing the PWM output's pulse width is performed. Note: This function should not be called directly, it will be invoked by the framework. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `node_id` OPC UA node id of the variable node representing the PWM output's pulse width.
<hr />

### `onWriteRequestPwmPulseWidth` <a id="classopcua_1_1_pwm_output_1a9a88eb5f43679c66d0be1288158e055c" class="anchor"></a>

```cpp
void onWriteRequestPwmPulseWidth(UA_Server * server, UA_NodeId const * pwm_pulse_width_node_id, uint32_t const pwm_pulse_width_us)
```

This function is called by the framework when a write-access on the variable node representing the PWM output's pulse width is performed. Note: This function should not be called directly, it will be invoked by the framework. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `pwm_pulse_width_node_id` OPC UA node id of the variable node representing the PWM output's pulse width. 

* `pwm_pulse_width_us` Desired PWM output pulse width in microseconds.
<hr />

### `SharedPtr` <a id="classopcua_1_1_pwm_output_1ad871619fdeeccd340fad668e1d59c668" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of a [PwmOutput](#classopcua_1_1_pwm_output) class.
<hr />

### `SetPwmFunc` <a id="classopcua_1_1_pwm_output_1a979bc7c17dbac4dfb518a65dd92c1c56" class="anchor"></a>

```cpp
typedef SetPwmFunc
```

SetPwmFunc is definition for a callback which is called during a write-access on the variable nodes representing PWM period and PWM pulse width subsequently configuring the PWN output with the desired period and pulse width.
<hr />

### `GetPwmPeriodFunc` <a id="classopcua_1_1_pwm_output_1ae9f4bd8023add25dda7eb038745e494c" class="anchor"></a>

```cpp
typedef GetPwmPeriodFunc
```

GetPwmPeriodFunc is definition for a callback which is called during a read-access on the variable node representing the PWM period and which is used to retrieve the actual PWM period.
<hr />

### `GetPwmPulseWidthFunc` <a id="classopcua_1_1_pwm_output_1ad9ca89b1b476fac49dd9d32306b819af" class="anchor"></a>

```cpp
typedef GetPwmPulseWidthFunc
```

GetPwmPulseWidthFunc is definition for a callback which is called during a read-access on the variable node representing the PWM pulse width and which is used to retrieve the actual PWM pulse width.
<hr />

### `create` <a id="classopcua_1_1_pwm_output_1a595c247e837f0e9a34e11b2b7b5a26ff" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const & parent_node_id, const char * display_name, SetPwmFunc const set_pwm_func, GetPwmPeriodFunc const get_pwm_period_func, GetPwmPulseWidthFunc const get_pwm_pulse_width_func)
```

Creates a new instance of the [opcua::PwmOutput](#classopcua_1_1_pwm_output) class, creating an OPC UA variable node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog output, i.e. "PWM Output P1". 

* `set_pwm_func` Function pointer which is called during a write-access on the variable node representing the PWM output and used for setting the PWM period and pulse width. 

* `get_pwm_period_func` Function pointer which is called during a read-access on the variable node representing the PWM output's period. 

* `get_pwm_pulse_width_func` Function pointer which is called during a read-access on the variable node representing the PWM output's pulse width. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::PwmOutput](#classopcua_1_1_pwm_output).
<hr />

# class `opcua::PwmOutputManager` <a id="classopcua_1_1_pwm_output_manager" class="anchor"></a>

Provides methods for creating and storing [PwmOutput](#classopcua_1_1_pwm_output) objects in a collection.

This class creates an OPC UA object node which is representing all PWM outputs of a given OPC UA devices. PWM outputs can be created and added to the OPC UA object node using this class.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`PwmOutputManager`](#classopcua_1_1_pwm_output_manager_1ae682f02d81ae9cdd8234e670e5140380) | Constructor of the [opcua::PwmOutputManager](#classopcua_1_1_pwm_output_manager) class. Note: Objects should not be instantiated directly, but by using [PwmOutputManager::create](#classopcua_1_1_pwm_output_manager_1a032bb88f9e5a54a6d68adc37a685374c).  |
| [`addPwmOutput`](#classopcua_1_1_pwm_output_manager_1a58a16587c5ea5b35d183031483d78009) | This function creates a new [opcua::PwmOutput](#classopcua_1_1_pwm_output) and adds it to the [opcua::PwmOutputManager](#classopcua_1_1_pwm_output_manager).  |
| [`SharedPtr`](#classopcua_1_1_pwm_output_manager_1ae595b9016fec91f1e5caa1a74df4bcd5) | SharedPtr is std::shared_ptr of an [PwmOutputManager](#classopcua_1_1_pwm_output_manager) class. |
| [`create`](#classopcua_1_1_pwm_output_manager_1a032bb88f9e5a54a6d68adc37a685374c) | Creates a new instance of the [opcua::PwmOutputManager](#classopcua_1_1_pwm_output_manager) class, creating an OPC UA object node abstraction in the process.  |

## Members

### `PwmOutputManager` <a id="classopcua_1_1_pwm_output_manager_1ae682f02d81ae9cdd8234e670e5140380" class="anchor"></a>

```cpp
PwmOutputManager(UA_NodeId const & node_id)
```

Constructor of the [opcua::PwmOutputManager](#classopcua_1_1_pwm_output_manager) class. Note: Objects should not be instantiated directly, but by using [PwmOutputManager::create](#classopcua_1_1_pwm_output_manager_1a032bb88f9e5a54a6d68adc37a685374c). 
#### Parameters
* `node_id` OPC UA node id of the object node representing all PWM outputs.
<hr />

### `addPwmOutput` <a id="classopcua_1_1_pwm_output_manager_1a58a16587c5ea5b35d183031483d78009" class="anchor"></a>

```cpp
void addPwmOutput(UA_Server * server, const char * display_name, PwmOutput::SetPwmFunc const set_pwm_func, PwmOutput::GetPwmPeriodFunc const get_pwm_period_func, PwmOutput::GetPwmPulseWidthFunc const get_pwm_pulse_width_func)
```

This function creates a new [opcua::PwmOutput](#classopcua_1_1_pwm_output) and adds it to the [opcua::PwmOutputManager](#classopcua_1_1_pwm_output_manager). 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing an analog output, i.e. "PWM Output P1". 

* `set_pwm_func` Function pointer which is called during a write-access on the variable node representing the PWM output and used for setting the PWM period and pulse width. 

* `get_pwm_period_func` Function pointer which is called during a read-access on the variable node representing the PWM output's period. 

* `get_pwm_pulse_width_func` Function pointer which is called during a read-access on the variable node representing the PWM output's pulse width.
<hr />

### `SharedPtr` <a id="classopcua_1_1_pwm_output_manager_1ae595b9016fec91f1e5caa1a74df4bcd5" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [PwmOutputManager](#classopcua_1_1_pwm_output_manager) class.
<hr />

### `create` <a id="classopcua_1_1_pwm_output_manager_1a032bb88f9e5a54a6d68adc37a685374c" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const parent_node_id)
```

Creates a new instance of the [opcua::PwmOutputManager](#classopcua_1_1_pwm_output_manager) class, creating an OPC UA object node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::PwmOutputManager](#classopcua_1_1_pwm_output_manager).
<hr />

# class `opcua::Relay` <a id="classopcua_1_1_relay" class="anchor"></a>

Provides an OPC UA variable node abstraction for controlling a mechanical or solid-state relay.

This class creates an OPC UA variable node which is representing a mechanical or solid-state relay. When writing to the OPC UA variable node a callback is triggered which either activates or deactivates the relay.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`Relay`](#classopcua_1_1_relay_1aee598064d5f606fcd74e8fe473af1232) | Constructor of the [opcua::Relay](#classopcua_1_1_relay) class. Note: Objects should not be instantiated directly, but by using [Relay::create](#classopcua_1_1_relay_1ae26e71efaed1d390c3c4f62e831fce0d).  |
| [`onWriteRequest`](#classopcua_1_1_relay_1aaea6db4d009ec591232c10d5ade7f861) | This function is called by the framework when a write-access on the variable node representing a relay is performed. Note: This function should not be called directly, it will be invoked by the framework.  |
| [`SharedPtr`](#classopcua_1_1_relay_1a28a191bf7c9a5d6b769d7662063f47d7) | SharedPtr is std::shared_ptr of a [Relay](#classopcua_1_1_relay) class. |
| [`OnSetRelayStateFunc`](#classopcua_1_1_relay_1a52b846933e1fb40082c3c76dba8b03ed) | OnSetRelayStateFunc is definition for a callback which is called during a write-access on the variable node representing a relay subsequently either activating (true -> switch closes) or deactivating a relay (false -> switch opens). |
| [`create`](#classopcua_1_1_relay_1ae26e71efaed1d390c3c4f62e831fce0d) | Creates a new instance of the [opcua::Relay](#classopcua_1_1_relay) class, creating an OPC UA variable node abstraction in the process.  |

## Members

### `Relay` <a id="classopcua_1_1_relay_1aee598064d5f606fcd74e8fe473af1232" class="anchor"></a>

```cpp
Relay(UA_NodeId const & node_id, OnSetRelayStateFunc const on_set_relay_state)
```

Constructor of the [opcua::Relay](#classopcua_1_1_relay) class. Note: Objects should not be instantiated directly, but by using [Relay::create](#classopcua_1_1_relay_1ae26e71efaed1d390c3c4f62e831fce0d). 
#### Parameters
* `node_id` OPC UA node id of the variable node representing the relay. 

* `on_set_relay_state` Function pointer which is called during a write-access on the variable node representing a relay.
<hr />

### `onWriteRequest` <a id="classopcua_1_1_relay_1aaea6db4d009ec591232c10d5ade7f861" class="anchor"></a>

```cpp
void onWriteRequest(UA_Server * server, UA_NodeId const * node_id, bool const value)
```

This function is called by the framework when a write-access on the variable node representing a relay is performed. Note: This function should not be called directly, it will be invoked by the framework. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `node_id` OPC UA node id of the variable node representing the relay. 

* `value` Desired state of the relay, true -> relay is activated (relay switch closes), false -> LED is deactivated (relay switch opens).
<hr />

### `SharedPtr` <a id="classopcua_1_1_relay_1a28a191bf7c9a5d6b769d7662063f47d7" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of a [Relay](#classopcua_1_1_relay) class.
<hr />

### `OnSetRelayStateFunc` <a id="classopcua_1_1_relay_1a52b846933e1fb40082c3c76dba8b03ed" class="anchor"></a>

```cpp
typedef OnSetRelayStateFunc
```

OnSetRelayStateFunc is definition for a callback which is called during a write-access on the variable node representing a relay subsequently either activating (true -> switch closes) or deactivating a relay (false -> switch opens).
<hr />

### `create` <a id="classopcua_1_1_relay_1ae26e71efaed1d390c3c4f62e831fce0d" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const & parent_node_id, const char * display_name, OnSetRelayStateFunc const on_set_relay_state)
```

Creates a new instance of the [opcua::Relay](#classopcua_1_1_relay) class, creating an OPC UA variable node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

* `display_name` Character string providing an easy identifiable name for the variable node representing a relay, i.e. "Relay 1". 

* `on_set_relay_state` Function pointer which is called during a write-access on the variable node representing a relay. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::Relay](#classopcua_1_1_relay).
<hr />

# class `opcua::RelayManager` <a id="classopcua_1_1_relay_manager" class="anchor"></a>

Provides methods for creating and storing [Relay](#classopcua_1_1_relay) objects in a collection.

This class creates an OPC UA object node which is representing all relays of a given OPC UA devices. Relays can be created and added to the OPC UA object node using this class.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`RelayManager`](#classopcua_1_1_relay_manager_1a9d4e2106b768a1f1fa0afd4a5056b962) | Constructor of the [opcua::RelayManager](#classopcua_1_1_relay_manager) class. Note: Objects should not be instantiated directly, but by using [RelayManager::create](#classopcua_1_1_relay_manager_1ae292144fb50a7f4b05e05f2520e41c21).  |
| [`addRelayOutput`](#classopcua_1_1_relay_manager_1a6dcce2b16d8f7f95ce6033b01e578297) | This function creates a new [opcua::Relay](#classopcua_1_1_relay) and adds it to the [opcua::RelayManager](#classopcua_1_1_relay_manager).  |
| [`SharedPtr`](#classopcua_1_1_relay_manager_1a5ed69275e13c134f54d16ba27eabc34c) | SharedPtr is std::shared_ptr of an [RelayManager](#classopcua_1_1_relay_manager) class. |
| [`create`](#classopcua_1_1_relay_manager_1ae292144fb50a7f4b05e05f2520e41c21) | Creates a new instance of the [opcua::RelayManager](#classopcua_1_1_relay_manager) class, creating an OPC UA object node abstraction in the process.  |

## Members

### `RelayManager` <a id="classopcua_1_1_relay_manager_1a9d4e2106b768a1f1fa0afd4a5056b962" class="anchor"></a>

```cpp
RelayManager(UA_NodeId const & node_id)
```

Constructor of the [opcua::RelayManager](#classopcua_1_1_relay_manager) class. Note: Objects should not be instantiated directly, but by using [RelayManager::create](#classopcua_1_1_relay_manager_1ae292144fb50a7f4b05e05f2520e41c21). 
#### Parameters
* `node_id` OPC UA node id of the object node representing all relays.
<hr />

### `addRelayOutput` <a id="classopcua_1_1_relay_manager_1a6dcce2b16d8f7f95ce6033b01e578297" class="anchor"></a>

```cpp
void addRelayOutput(UA_Server * server, const char * display_name, Relay::OnSetRelayStateFunc const on_set_relay_state)
```

This function creates a new [opcua::Relay](#classopcua_1_1_relay) and adds it to the [opcua::RelayManager](#classopcua_1_1_relay_manager). 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node representing a relay, i.e. "Relay 1". 

* `on_set_relay_state` Function pointer which is called during a write-access on the variable node representing a relay.
<hr />

### `SharedPtr` <a id="classopcua_1_1_relay_manager_1a5ed69275e13c134f54d16ba27eabc34c" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [RelayManager](#classopcua_1_1_relay_manager) class.
<hr />

### `create` <a id="classopcua_1_1_relay_manager_1ae292144fb50a7f4b05e05f2520e41c21" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, UA_NodeId const parent_node_id)
```

Creates a new instance of the [opcua::RelayManager](#classopcua_1_1_relay_manager) class, creating an OPC UA object node abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `parent_node_id` OPC UA node id of parent object in OPC UA tree. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::RelayManager](#classopcua_1_1_relay_manager).
<hr />

# class `opcua::Opta` <a id="classopcua_1_1_opta" class="anchor"></a>

Provides functions for exposing various IO capabilities of the Arduino [Opta](#classopcua_1_1_opta) via OPC UA.

This class allows the user to expose analog and digital inputs, as well as relay and LED outputs via OPC UA properties.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`Opta`](#classopcua_1_1_opta_1a283e8bca4e4044cce2e85d86bb95cd55) | Constructor of the [opcua::Opta](#classopcua_1_1_opta) class.  |
| [`addAnalogInput`](#classopcua_1_1_opta_1ae315b43f05cb1e93bf93f5c8372b2d9b) | Adds an analog input to the [opcua::Opta](#classopcua_1_1_opta) object node and exposes it as an OPC UA variable node.  |
| [`addDigitalInput`](#classopcua_1_1_opta_1a423a610eaf81a7a216471b74494b8b6a) | Adds a digital input to the [opcua::Opta](#classopcua_1_1_opta) object node and exposes it as an OPC UA variable node.  |
| [`addRelayOutput`](#classopcua_1_1_opta_1aa291d0f49cd9ecdedd21ae589fa7b3ce) | Adds a relay output to the [opcua::Opta](#classopcua_1_1_opta) object node and exposes it as an OPC UA variable node.  |
| [`addLedOutput`](#classopcua_1_1_opta_1ad02dbe76b766e9e3cc037acd40659c19) | Adds a LED output to the [opcua::Opta](#classopcua_1_1_opta) object node and exposes it as an OPC UA variable node.  |
| [`SharedPtr`](#classopcua_1_1_opta_1a75b2e05ac5b273859709490686dfa165) | SharedPtr is std::shared_ptr of an [Opta](#classopcua_1_1_opta) class. |
| [`create`](#classopcua_1_1_opta_1a01f8da6cd187c6237c903dc6d004828a) | Creates a new instance of the [opcua::Opta](#classopcua_1_1_opta) class, creating an OPC UA object abstraction in the process.  |

## Members

### `Opta` <a id="classopcua_1_1_opta_1a283e8bca4e4044cce2e85d86bb95cd55" class="anchor"></a>

```cpp
Opta(UA_Server * server, UA_NodeId const & node_id, OptaVariant::Type const opta_type)
```

Constructor of the [opcua::Opta](#classopcua_1_1_opta) class. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `node_id` OPC UA node id uniquely identifying the OPC UA representation of the [opcua::Opta](#classopcua_1_1_opta) class as an OPC UA object node. 

* `opta_type` Enumerated type describing which [Opta](#classopcua_1_1_opta) (WiFi, RS485, Lite) variant is being created.
<hr />

### `addAnalogInput` <a id="classopcua_1_1_opta_1ae315b43f05cb1e93bf93f5c8372b2d9b" class="anchor"></a>

```cpp
void addAnalogInput(UA_Server * server, const char * display_name, AnalogInput::OnReadRequestFunc const on_read_request)
```

Adds an analog input to the [opcua::Opta](#classopcua_1_1_opta) object node and exposes it as an OPC UA variable node. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node. 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the analog input.
<hr />

### `addDigitalInput` <a id="classopcua_1_1_opta_1a423a610eaf81a7a216471b74494b8b6a" class="anchor"></a>

```cpp
void addDigitalInput(UA_Server * server, const char * display_name, DigitalInput::OnReadRequestFunc const on_read_request)
```

Adds a digital input to the [opcua::Opta](#classopcua_1_1_opta) object node and exposes it as an OPC UA variable node. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node. 

* `on_read_request` Function pointer which is called during a read-access on the variable node representing the digital input.
<hr />

### `addRelayOutput` <a id="classopcua_1_1_opta_1aa291d0f49cd9ecdedd21ae589fa7b3ce" class="anchor"></a>

```cpp
void addRelayOutput(UA_Server * server, const char * display_name, Relay::OnSetRelayStateFunc const on_set_relay_state)
```

Adds a relay output to the [opcua::Opta](#classopcua_1_1_opta) object node and exposes it as an OPC UA variable node. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node. 

* `on_set_relay_state` Function pointer which is called during a write-access on the variable node representing the relay output.
<hr />

### `addLedOutput` <a id="classopcua_1_1_opta_1ad02dbe76b766e9e3cc037acd40659c19" class="anchor"></a>

```cpp
void addLedOutput(UA_Server * server, const char * display_name, Led::OnSetLedStateFunc const on_set_led_state)
```

Adds a LED output to the [opcua::Opta](#classopcua_1_1_opta) object node and exposes it as an OPC UA variable node. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `display_name` Character string providing an easy identifiable name for the variable node. 

* `on_set_led_state` Function pointer which is called during a write-access on the variable node representing the LED output.
<hr />

### `SharedPtr` <a id="classopcua_1_1_opta_1a75b2e05ac5b273859709490686dfa165" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [Opta](#classopcua_1_1_opta) class.
<hr />

### `create` <a id="classopcua_1_1_opta_1a01f8da6cd187c6237c903dc6d004828a" class="anchor"></a>

```cpp
static SharedPtr create(UA_Server * server, OptaVariant::Type const opta_type)
```

Creates a new instance of the [opcua::Opta](#classopcua_1_1_opta) class, creating an OPC UA object abstraction in the process. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

* `opta_type` Enumerated type describing which [Opta](#classopcua_1_1_opta) (WiFi, RS485, Lite) variant is being created. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::Opta](#classopcua_1_1_opta).
<hr />

# class `opcua::OptaExpansionManager` <a id="classopcua_1_1_opta_expansion_manager" class="anchor"></a>

High-level class to create OPC UA representations for Arduino [Opta](#classopcua_1_1_opta) digital and analog expansion boards.

This class allows the user to create OPC UA representations of digital (mechanical and solid-state relays) as well as analog expansion boards connected to the Arduino [Opta](#classopcua_1_1_opta).

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`OptaExpansionManager`](#classopcua_1_1_opta_expansion_manager_1a50f8f17012161470f0c80efa83f8aba8) | Constructor of the [opcua::OptaExpansionManager](#classopcua_1_1_opta_expansion_manager) class.  |
| [`createDigitalMechanicalExpansion`](#classopcua_1_1_opta_expansion_manager_1a9a67371ac51ab2b1d13850576345096f) | Creates a new instance of the [opcua::DigitalMechExpansion](#classopcua_1_1_digital_mech_expansion) (digital expansion with mechanical relays) class.  |
| [`createDigitalSolidStateExpansion`](#classopcua_1_1_opta_expansion_manager_1ab13e39fc5dbfe377d1a8c7a0f4dc2169) | Creates a new instance of the [opcua::DigitalStSolidExpansion](#classopcua_1_1_digital_st_solid_expansion) (digital expansion with solid-state relays) class.  |
| [`createAnalogExpansion`](#classopcua_1_1_opta_expansion_manager_1a25cb57fd1d37dc0c08f89a9eb5a70b14) | Creates a new instance of the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class.  |
| [`SharedPtr`](#classopcua_1_1_opta_expansion_manager_1abd9459852a863117520d564b018779dd) | SharedPtr is std::shared_ptr of an [OptaExpansionManager](#classopcua_1_1_opta_expansion_manager) class. |
| [`create`](#classopcua_1_1_opta_expansion_manager_1a1fe5f600c6a428877e5c3fc2529a1db6) | Creates a new instance of the [opcua::OptaExpansionManager](#classopcua_1_1_opta_expansion_manager) class.  |

## Members

### `OptaExpansionManager` <a id="classopcua_1_1_opta_expansion_manager_1a50f8f17012161470f0c80efa83f8aba8" class="anchor"></a>

```cpp
inline OptaExpansionManager(UA_Server * server)
```

Constructor of the [opcua::OptaExpansionManager](#classopcua_1_1_opta_expansion_manager) class. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library.
<hr />

### `createDigitalMechanicalExpansion` <a id="classopcua_1_1_opta_expansion_manager_1a9a67371ac51ab2b1d13850576345096f" class="anchor"></a>

```cpp
DigitalMechExpansion::SharedPtr createDigitalMechanicalExpansion(uint8_t const exp_num)
```

Creates a new instance of the [opcua::DigitalMechExpansion](#classopcua_1_1_digital_mech_expansion) (digital expansion with mechanical relays) class. 
#### Parameters
* `exp_num` A numerical identifier provided by the Arduino_Opta_Blueprint library and identifying the number of the expansion module in the daisy-chain of expansion modules, i.e. exp_num = 2 refers to the second connect expansion module. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::DigitalMechExpansion](#classopcua_1_1_digital_mech_expansion).
<hr />

### `createDigitalSolidStateExpansion` <a id="classopcua_1_1_opta_expansion_manager_1ab13e39fc5dbfe377d1a8c7a0f4dc2169" class="anchor"></a>

```cpp
DigitalStSolidExpansion::SharedPtr createDigitalSolidStateExpansion(uint8_t const exp_num)
```

Creates a new instance of the [opcua::DigitalStSolidExpansion](#classopcua_1_1_digital_st_solid_expansion) (digital expansion with solid-state relays) class. 
#### Parameters
* `exp_num` A numerical identifier provided by the Arduino_Opta_Blueprint library and identifying the number of the expansion module in the daisy-chain of expansion modules, i.e. exp_num = 2 refers to the second connect expansion module. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::DigitalStSolidExpansion](#classopcua_1_1_digital_st_solid_expansion).
<hr />

### `createAnalogExpansion` <a id="classopcua_1_1_opta_expansion_manager_1a25cb57fd1d37dc0c08f89a9eb5a70b14" class="anchor"></a>

```cpp
AnalogExpansion::SharedPtr createAnalogExpansion(uint8_t const exp_num)
```

Creates a new instance of the [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion) class. 
#### Parameters
* `exp_num` A numerical identifier provided by the Arduino_Opta_Blueprint library and identifying the number of the expansion module in the daisy-chain of expansion modules, i.e. exp_num = 2 refers to the second connect expansion module. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::AnalogExpansion](#classopcua_1_1_analog_expansion).
<hr />

### `SharedPtr` <a id="classopcua_1_1_opta_expansion_manager_1abd9459852a863117520d564b018779dd" class="anchor"></a>

```cpp
typedef SharedPtr
```

SharedPtr is std::shared_ptr of an [OptaExpansionManager](#classopcua_1_1_opta_expansion_manager) class.
<hr />

### `create` <a id="classopcua_1_1_opta_expansion_manager_1a1fe5f600c6a428877e5c3fc2529a1db6" class="anchor"></a>

```cpp
inline static SharedPtr create(UA_Server * server)
```

Creates a new instance of the [opcua::OptaExpansionManager](#classopcua_1_1_opta_expansion_manager) class. 
#### Parameters
* `server` Pointer to the OPC UA server implementation of the open62541 library. 

#### Returns
std::shared_ptr holding the newly allocated instance of [opcua::OptaExpansionManager](#classopcua_1_1_opta_expansion_manager).
<hr />

# class `opcua::OptaVariant` <a id="classopcua_1_1_opta_variant" class="anchor"></a>

Enables determination of [Opta](#classopcua_1_1_opta) variant (WiFi, RS485, Lite) on which the OPC UA firmware is running on.

This class allows the user to determine the concrete [Opta](#classopcua_1_1_opta) variant (WiFi, RS485, Lite) on which the OPC UA firmware is running on. This is archived by interrogating information stored by the bootloader.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`OptaVariant`](#classopcua_1_1_opta_variant_1acaaa63b7b37af5fab4f3df0080669d76) | The constructor of [OptaVariant](#classopcua_1_1_opta_variant) is deleted because this object shall not be instantiated. |
| [`OptaVariant`](#classopcua_1_1_opta_variant_1a82a5e186512309818c5a4922e18097b6) | The copy constructor of [OptaVariant](#classopcua_1_1_opta_variant) is deleted because this object shall not be copied. |
| [`Type`](#classopcua_1_1_opta_variant_1abd647955f6c174916e3c0e25ed240ea8) | Type is an enumeration type to describe the various different Arduino [Opta](#classopcua_1_1_opta) variants. |
| [`getOptaVariant`](#classopcua_1_1_opta_variant_1ae3b4401cb74c9cc418b9a225cd611afd) | Determines the current [Opta](#classopcua_1_1_opta) variant by reading information provided by the bootloader.  |
| [`toString`](#classopcua_1_1_opta_variant_1aa2b41d503d530c8f135f420b54d94068) | Convert enumerated variant type to variant product name.  |
| [`toSKUString`](#classopcua_1_1_opta_variant_1aba13996161689f6feb65753daa4e796e) | Convert enumerated variant type to variant product SKU number.  |

## Members

### `OptaVariant` <a id="classopcua_1_1_opta_variant_1acaaa63b7b37af5fab4f3df0080669d76" class="anchor"></a>

```cpp
OptaVariant() = delete
```

The constructor of [OptaVariant](#classopcua_1_1_opta_variant) is deleted because this object shall not be instantiated.
<hr />

### `OptaVariant` <a id="classopcua_1_1_opta_variant_1a82a5e186512309818c5a4922e18097b6" class="anchor"></a>

```cpp
OptaVariant( OptaVariant const &) = delete
```

The copy constructor of [OptaVariant](#classopcua_1_1_opta_variant) is deleted because this object shall not be copied.
<hr />

### `Type` <a id="classopcua_1_1_opta_variant_1abd647955f6c174916e3c0e25ed240ea8" class="anchor"></a>

```cpp
enum Type
```

 Values                         | Descriptions                                
--------------------------------|---------------------------------------------
Lite            | Arduino [Opta](#classopcua_1_1_opta) Lite
RS485            | Arduino [Opta](#classopcua_1_1_opta) RS485
WiFi            | Arduino [Opta](#classopcua_1_1_opta) WiFi

Type is an enumeration type to describe the various different Arduino [Opta](#classopcua_1_1_opta) variants.
<hr />

### `getOptaVariant` <a id="classopcua_1_1_opta_variant_1ae3b4401cb74c9cc418b9a225cd611afd" class="anchor"></a>

```cpp
static bool getOptaVariant( Type & type)
```

Determines the current [Opta](#classopcua_1_1_opta) variant by reading information provided by the bootloader. 
#### Parameters
* `type` Output parameter containing the current [Opta](#classopcua_1_1_opta) variant. 

#### Returns
True if the [Opta](#classopcua_1_1_opta) variant could be obtained successfully.
<hr />

### `toString` <a id="classopcua_1_1_opta_variant_1aa2b41d503d530c8f135f420b54d94068" class="anchor"></a>

```cpp
static std::string toString( Type const type)
```

Convert enumerated variant type to variant product name. 
#### Parameters
* `type` Enumerated type describing an [Opta](#classopcua_1_1_opta) variant. 

#### Returns
String describing the [Opta](#classopcua_1_1_opta) variant's product name, i.e. [Type::Lite](#classopcua_1_1_opta_variant_1abd647955f6c174916e3c0e25ed240ea8a2dc5b73e350ccaf3a027e503e0220979) -> "Arduino Opta Lite"
<hr />

### `toSKUString` <a id="classopcua_1_1_opta_variant_1aba13996161689f6feb65753daa4e796e" class="anchor"></a>

```cpp
static std::string toSKUString( Type const type)
```

Convert enumerated variant type to variant product SKU number. 
#### Parameters
* `type` Enumerated type describing an [Opta](#classopcua_1_1_opta) variant. 

#### Returns
String describing the [Opta](#classopcua_1_1_opta) variant's SKU number, i.e. [Type::Lite](#classopcua_1_1_opta_variant_1abd647955f6c174916e3c0e25ed240ea8a2dc5b73e350ccaf3a027e503e0220979) -> "AFX00003"
<hr />

# class `opcua::NTPUtils` <a id="classopcua_1_1_n_t_p_utils" class="anchor"></a>

Provides a method for obtaining the current time via UDP.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`getTime`](#classopcua_1_1_n_t_p_utils_1afb634cf860f54e4cc7d1cc23073af912) | Obtain the current UTC time via NTP.  |

## Members

### `getTime` <a id="classopcua_1_1_n_t_p_utils_1afb634cf860f54e4cc7d1cc23073af912" class="anchor"></a>

```cpp
static unsigned long getTime(UDP & udp)
```

Obtain the current UTC time via NTP. 
#### Parameters
* `udp` Reference to UDP network socket. 

#### Returns
current UTC time in seconds since Epoch (Unix Time)
<hr />

# class `opcua::ExpansionType` <a id="classopcua_1_1_expansion_type" class="anchor"></a>

Provides a method for converting Arduino_Opta_Blueprint's ExpansionType_t to a human readable string.

## Summary

 Members                        | Descriptions                                
--------------------------------|---------------------------------------------
| [`ExpansionType`](#classopcua_1_1_expansion_type_1a23ebe20fe86bce334acbebca12453f97) | The constructor of [ExpansionType](#classopcua_1_1_expansion_type) is deleted because this object shall not be instantiated. |
| [`ExpansionType`](#classopcua_1_1_expansion_type_1a93119a7176cebcc818038cc2cb5bd566) | The copy constructor of [ExpansionType](#classopcua_1_1_expansion_type) is deleted because this object shall not be copied. |
| [`toStr`](#classopcua_1_1_expansion_type_1a34fa70ff71feda985dd570b7e25f2286) | Converts Arduino_Opta_Blueprint's ExpansionType_t into a human readable string.  |

## Members

### `ExpansionType` <a id="classopcua_1_1_expansion_type_1a23ebe20fe86bce334acbebca12453f97" class="anchor"></a>

```cpp
ExpansionType() = delete
```

The constructor of [ExpansionType](#classopcua_1_1_expansion_type) is deleted because this object shall not be instantiated.
<hr />

### `ExpansionType` <a id="classopcua_1_1_expansion_type_1a93119a7176cebcc818038cc2cb5bd566" class="anchor"></a>

```cpp
ExpansionType( ExpansionType const &) = delete
```

The copy constructor of [ExpansionType](#classopcua_1_1_expansion_type) is deleted because this object shall not be copied.
<hr />

### `toStr` <a id="classopcua_1_1_expansion_type_1a34fa70ff71feda985dd570b7e25f2286" class="anchor"></a>

```cpp
inline static std::string toStr(ExpansionType_t const type)
```

Converts Arduino_Opta_Blueprint's ExpansionType_t into a human readable string. 
#### Parameters
* `type` Numeric identifier identifying the type of connected expansion board. 

#### Returns
Human readable string describing the concrete expansion module.
<hr />

